#!/usr/bin/env python3

"""Reads airport data from an csv-file in the format supplied for
the public domain at ** http://ourairports.com/data/airports.csv **
and outputs converted data into a sqlite3 database needed by the
adnite2 (or a similar) night vision program suite.

* If there is no database, the database will be created.
  If the database exists already, pre existing entries will be replaced,
  others remain untouched.

* Location indicators are converted (from Unicode) to ASCII

* The (long) station name consists of municipality + airport name
  (if municipality is given), otherwise of the airport name alone.
  The name is converted to ASCII and truncated to a reasonable
  maximum length. Truncated names are indicated by trailing '..'.

* Latitude and longitude is converted from decimal arc degrees to
  arc deg., arc minute and arc second where the sign of degrees
  is also shifted to to the sign of minutes and seconds
  (e.g. -13.52 deg --> -13 deg  -31 min  -12 sec)

* Station height is converted from foot to meters
"""

from inspect import currentframe, getframeinfo
import argparse
import csv
import sqlite3
import sys
import unidecode  # https://pypi.python.org/pypi/Unidecode

import math as m
import os.path as op


__version__ = "0.1.0dev201600401"
__copyright__ = """(c) Dietmar Thaler, 2016
E: dietmar.thaler@posteo.at

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

For details see http://www.gnu.org/licenses/gpl-2.0.html"""


METER_PER_FOOT = 0.3048
FOOT_PER_METER = 1. / METER_PER_FOOT

INPUTCSV = './our-airports.csv'
DBNAME = './station_db.sqlite'


DEF_TRUNC_NAME = 50  # default maximum length for the station name
MAX_TRUNC_NAME = 80  # maximum length for the station name
MIN_TRUNC_NAME = 20  # minimum length for the station name
LINEFORMAT = "{0:10s} {1:_XXX_s}  {2:1s}{3:3.0f} {4:2.0f} {5:2.0f}   {6:1s}{7:3.0f} {8:2.0f} {9:2.0f} {10:5.0f}"

CREATEDB = """CREATE TABLE "stationlist" """
CREATEDB = CREATEDB + """("indicator" TEXT PRIMARY KEY  NOT NULL """
CREATEDB = CREATEDB + ""","name" TEXT,"latd" INTEGER,"latm" INTEGER,"lats" """
CREATEDB = CREATEDB + \
    """INTEGER,"lond" INTEGER,"lonm" INTEGER,"lons" INTEGER,"elev" FLOAT)"""
INSERTDB = "INSERT OR REPLACE INTO stationlist VALUES ( ? , ? , ? , ? ,? , ? , ? , ? , ? )"


def create_table(dbname):
    conn = sqlite3.connect(dbname)
    c = conn.cursor()
    c.execute(CREATEDB)
    conn.commit()
    conn.close()


def deg2dms(deg, plus='+', minus='-'):
    """Takes decimal degrees as argument. Returns the integer part of arc deg as dd,
    arc minutes as mm, and arc seconds as ss. The sign is put into the
    int sign (+/-1) and the sign character ('+'. '-'). dd, mm and ss is positve
    definite. Optional arguments plus and minus give the character
    for positive and negative sign that default to + and -"""
    sign = round(m.copysign(1, deg * 1.0))
    deg = abs(deg)
    dd = int(deg)
    m1 = (deg - dd) * 60.
    mm = int(m1)
    ss = int(round((m1 - mm) * 60.))
    if sign == -1:
        signchar = minus
    else:
        signchar = plus
    if ss == 60:
        ss = 0
        mm = mm + 1
    if mm == 60:
        mm = 0
        dd = dd + 1
    return sign, signchar, dd, mm, ss


def dms2deg(dd, mm=0, ss=0):
    """Returns decimal degrees as function of degrees dd, arc minutes mm
    and arc seconds ss. To avoid ambiguities the sign of each component has
    do be the same. Otherwise an error is returned.
    e.g.: (dd>=0, mm>=0 ss>=0) or (dd<=0 mm<=0 ss<=0)"""
    rangeok = (mm >= -60) and (mm <= 60) and (ss >= -60) and (ss <= 60)
    signok = (
        dd >= 0 and mm >= 0 and ss >= 0) or (
        dd <= 0 and mm <= 0 and ss <= 0)
    if signok and rangeok:
        return dd + mm / 60. + ss / 3600.
    else:
        if not signok:
            frameinfo = getframeinfo(currentframe())
            estr = 'ERROR in {0}.ddmmss2deg\nin linenno. {1}: dd, mm, ss are not of the same sign: {2} {3} {4}'
            return sys.exit(
                estr.format(frameinfo.filename, frameinfo.lineno, dd, mm, ss))
        if not rangeok:
            frameinfo = getframeinfo(currentframe())
            estr = 'ERROR in {0}.ddmmss2deg\nin linenno. {1}: mm, ss not within +/- 60: {2},{3}'
            return sys.exit(
                estr.format(frameinfo.filename, frameinfo.lineno, mm, ss))


def insert_table(
        sqlitecursor, loc, name, latd, latm, lats, lond, lonm, lons, elev):
    """Insert data line (without commit to speed up"""
    sqlitecursor.execute(
        INSERTDB,
        (loc,
         name,
         latd,
         latm,
         lats,
         lond,
         lonm,
         lons,
         elev))


def ddmmss2deg(dd, mm=0, ss=0):
    """Returns decimal dgress as function of degress dd, arc minutes mm
    and arc seconds ss, where only dd has got a sign. The positive definite
    values of mm and ss are corrected for the sign of dd."""
    m1 = m.copysign(mm, dd)
    s1 = m.copysign(ss, dd)
    return dd + m1 / 60. + s1 / 3600.


def trunc_str(s, length=80, ellipse='..'):
    """Truncate the string s to lenght length including the ellipse. If the string
    is shorter den length then do nothing."""
    le = len(ellipse)
    ls = length - le
    return s[:ls] + (s[ls:] and ellipse)


def main():
    try:
        create_table(dbname)
        print('Create database {0} - continue'.format(dbname))
    except sqlite3.Error:
        print('Database {0} alredy exists - continue!'.format(dbname))

    conn = sqlite3.connect(dbname)
    cursor = conn.cursor()
    with open(inputcsv) as csvfile:
        reader = csv.DictReader(csvfile)
        num_ok = 0
        num_failed = 0
        for row in reader:
            station = unidecode.unidecode(row['ident'].strip())
            muni = unidecode.unidecode(row['municipality'].strip())
            name0 = unidecode.unidecode(row['name'].strip())
            if len(muni) > 0:
                name = muni + ' - ' + name0
            else:
                name = name0
            name = trunc_str(name, truncname)
            try:
                latdeg = float(row['latitude_deg'])
                londeg = float(row['longitude_deg'])
                try:
                    elev = int(
                        round(
                            float(
                                row['elevation_ft']) *
                            METER_PER_FOOT))
                except ValueError:
                    elev = -999.9
                lasign, lasignchar, lad, lam, las = deg2dms(latdeg)
                losign, losignchar, lod, lom, los = deg2dms(londeg)
                s = lineformat.format(station, name,
                                      lasignchar, lad, lam, las,
                                      losignchar, lod, lom, los, elev)
                print(s)
                lad = lad * lasign
                lam = lam * lasign
                las = las * lasign
                lod = lod * losign
                lom = lom * losign
                los = los * losign
                num_ok += 1
                insert_table(
                    cursor,
                    station,
                    name,
                    lad,
                    lam,
                    las,
                    lod,
                    lom,
                    los,
                    elev)
            except ValueError:
                num_failed += 1
                pass
        conn.commit()
        conn.close()
        print(100 * '-')
        print(num_ok, ' stations inserted')
        print(num_failed, ' stations ommited')

# ================================================================
TRUNCHELP = """maximum length of long station names, defaults to {0},
limited to the range {1}..{2}"""\
    .format(DEF_TRUNC_NAME, MIN_TRUNC_NAME, MAX_TRUNC_NAME)
parser = argparse.ArgumentParser(description=__doc__,
                                 formatter_class=argparse.RawTextHelpFormatter)
parser.add_argument(
    '-i',
    '--inputcsv',
    required=True,
    help='Input csv-file with airport data')
parser.add_argument(
    '-o',
    '--outputdb',
    required=True,
    help='Output sqlite3 database file.')
parser.add_argument('-t', '--truncate',
                    type=int, default=DEF_TRUNC_NAME,
                    help=TRUNCHELP)
parser.add_argument('-c', '--copyright', help='Display copyright notice and exit',
                    action='store_true')
parser.add_argument(
    '--version',
    action='version',
    version='%(prog)s ' +
    __version__)

args = parser.parse_args()
if args.copyright:
    print('\n' + op.basename(sys.argv[0]) + ' - Version ' + __version__)
    print(__copyright__)
else:
    dbname = args.outputdb
    inputcsv = args.inputcsv
    truncname = min(max(args.truncate, MIN_TRUNC_NAME), MAX_TRUNC_NAME)
    lineformat = LINEFORMAT.replace('_XXX_', str(truncname))

if __name__ == '__main__':
    sys.exit(main())
