# Created 2016-03-15
# Modified 2017-04-15

"""Simple tool to insert and delete_station private stations (location indicator
starting with underbar '_') into the local adnite station databases. Editing of
stations not starting with '_' is prohibited.

Stations in the data bank can be listed using search patterns with wild cards.

"""

import argparse
import datetime
import os
import re
import sys

from adnite2.dattims import check_ddmmss, sign_of_dms
from adnite2.db import insert_replace_station, delete_station, read_station_list_by_wildcard
import adnite2.ToolTip as ToolTip
import adnite2.constants as constants
import os.path as op
import tkinter as tk
import tkinter.filedialog as tkfd
import tkinter.font as tkfont
import tkinter.messagebox as tkmb
import tkinter.scrolledtext as st
import tkinter.ttk as ttk


# python3
# Declarative stuff
__progname__ = op.basename(sys.argv[0])
__pyversion___ = "{0}.{1}.{2}".format(
                                    sys.version_info[0],
                                    sys.version_info[1],
                                    sys.version_info[2])
__copyright__ = '© Dietmar Thaler, 2016-2017'
# version__ = '0.1.dev20160315'
# __version__ = '0.2.dev20170415'
__version__ = '0.2.dev20170422'
__license__ = 'EUPL V1.1 https://joinup.ec.europa.eu/software/page/eupl/licence-eupl'


TITLE0 = "Adnite2DBedit"
TITLE1 = "Add/delete 'private' stations to/from database and list members"
ABOUTTXT = TITLE0 + '\n' + \
    '(Python ' + __pyversion___ + ')\n\n' + TITLE1 + '\n'
ABOUTTXT = ABOUTTXT + __copyright__ + '\n'
ABOUTTXT = ABOUTTXT + 'Version ' + __version__ + '\n\n'
ABOUTTXT = ABOUTTXT + 'License: ' + __license__

TIP_STATION = """Insert a 'private' location indicator.\n
The first character MUST be an underscore '_' to indicate the non official character of the station.
\n The next character must be alphabetic. Other alphanumerics may follow.
\nMinimum length 2, Maximum 10 characters.
"""
TIP_STATPATTERN = """List stations according to pattern. \n
Wildcards:\n
'*' .. any character of any number\n
'?' .. one single character. \n
ATTENTION: Searches both, in location indicators and (long) station names.
It is case insensitive (e.g. 'A*b' == 'a*B')"""
TIP_DEGREES = """Insert LATITUDE or LONGITUDE as
+/-dd +/-mm +/-ss\n
Positive values denote 'North' latitude or 'East' longitude, respectively.\n
Negative values denote 'South' latitude or 'West' longitude, respectively.\n
ATTENTION: the sign (+/-) must be given simultaneously in dd, mm and ss.\n
EXAMPLES:\n
  0 -30 -30  --> OK
  0 -30  30  --> ERROR
-13 -01 -02  --> OK
-13 -01  02  --> ERROR"""

FONT = 'Arial'
FONTSIZE0 = 13
FONTSIZE1 = 10
FONTSIZE2 = 9
# FONTSIZE3 = FONTSIZE2  # historical named variable
COLWIDTH0 = 30
COLWIDTH1 = 40
BUTTONWIDTH = 12
BOLD = "bold"
NORMAL = "normal"
MAXCOLUMN = 8  # 10
PADX = PADY = 5
TXT_MAX_X = 100
TXT_MAX_Y = 16

RESTATION = r'^_[a-zA-Z0-9]{1,9}$'

# Try to read fontsizes from adnite2-config-file and overwrite hard coded defaults
# (if possible. let defaults if anything goes wrong)
configfilen = constants.USER_CONFIGFILENAME
configparms = constants.read_parameters(
    configfilen,
    allowedstrings=constants.CONFIG_PARAM_STRINGS,
    allowedlists=constants.CONFIG_PARAM_LISTS)
try:
    FONTSIZE0 = configparms['fontsize0']
    FONTSIZE1 = configparms['fontsize1']
    FONTSIZE2 = configparms['fontsize2']
except KeyError:
    FONTSIZE0 = 13
    FONTSIZE1 = 9
    FONTSIZE2 = 8
    print('Warning - adnitedbedit: Font Size set to defaults')
    print('Set fontsize0, fontsize1, fontsize2 in ~/.adnite2/default.cfg')
    print(FONTSIZE0, FONTSIZE1, FONTSIZE2)


def trunc_str(s, length=80, ellipse='..'):
    """Truncate the string s to lenght length including the ellipse. If the string
    is shorter den length then do nothing."""
    le = len(ellipse)
    ls = length - le
    return s[:ls] + (s[ls:] and ellipse)


def log_message(logf, txt, message):
    """Write messages to the logfile logf and the scrolledtextwindow txt"""
    nun = datetime.datetime.utcnow().strftime("%H:%M:%S: ")
    fullm = TXT_MAX_X * '-' + "\n"
    fullm = fullm + 'AdniteDBedit ' + nun + message + '\n'
    # status area
    txt.configure(state=tk.NORMAL)
    txt.insert(tk.END, fullm)
    txt.see(tk.END)
    txt.configure(state=tk.DISABLED)
    # log file
    try:
        print(fullm, end='', file=logf)
    except UnicodeEncodeError as e:
        print(e, file=logf)
        print('Continue ..', file=logf)
    logf.flush()
    return


class AdniteDB:

    def __init__(self, root):
        global database
        # Set the default font size to FONTSIZE2
        default_font = tkfont.nametofont("TkDefaultFont")
        default_font.configure(size=FONTSIZE2)
        root.option_add("*Font", default_font)
        root.protocol('WM_DELETE_WINDOW', self.do_exit)
        root.resizable(width=False, height=False)
        self.root = root

        self.logfile = sys.stdout
        self.scr_txt = st.ScrolledText(
            master=root,
            wrap=tk.WORD,
            width=TXT_MAX_X,
            height=TXT_MAX_Y,
            state=tk.NORMAL,
            font=('Mono', '9', 'normal'),
            background='lightgrey')
        self.scr_txt.configure(state=tk.DISABLED)
        log_message(self.logfile, self.scr_txt, ABOUTTXT)

        self.v_station = tk.StringVar()
        self.v_station.set('_NIEWO')
        self.v_name = tk.StringVar()
        self.v_name.set('In der Pampa')
        self.v_latdeg = tk.StringVar()
        self.v_latdeg.set('0')
        self.v_latmin = tk.StringVar()
        self.v_latmin.set('0')
        self.v_latsec = tk.StringVar()
        self.v_latsec.set('0')
        self.v_londeg = tk.StringVar()
        self.v_londeg.set('0')
        self.v_lonmin = tk.StringVar()
        self.v_lonmin.set('0')
        self.v_lonsec = tk.StringVar()
        self.v_lonsec.set('0')
        self.v_elevation = tk.StringVar()
        self.v_elevation.set('0')
        self.v_statpat = tk.StringVar()
        self.v_statpat.set('LO??')

        # WIDGETS
        l_title = tk.Label(text=TITLE1, font=(FONT, FONTSIZE0),
                           width=COLWIDTH0)
        self.button_about = tk.Button(self.root, text="About",
                                      width=BUTTONWIDTH,
                                      command=self.about)
        self.button_help = tk.Button(self.root, text='Help',
                                     width=BUTTONWIDTH,
                                     command=self.help)
        self.l_database = tk.Label(text='Data-Base: ' + database,
                                   font=(FONT, FONTSIZE1, BOLD))
        self.l_station = tk.Label(text='Station (2..10 chars): ',
                                  font=(FONT, FONTSIZE1, BOLD),
                                  justify=tk.LEFT)
        self.l_xstation = tk.Label(
            text="start with '_'",
            font=(FONT, FONTSIZE1, NORMAL), justify=tk.LEFT)
        self.e_station = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                  bg='white', width=14,
                                  textvariable=self.v_station,
                                  validate="focusout",
                                  validatecommand=self.onIndicValidate)
        self.l_name = tk.Label(text='Long station name: ',
                               font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT)
        self.e_name = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                               bg='white',
                               width=50,
                               textvariable=self.v_name)
        self.l_lat = tk.Label(text='Latitude (+/-dd mm ss): ',
                              font=(FONT, FONTSIZE1, BOLD),
                              justify=tk.LEFT)
        self.e_lat_deg = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                  bg='white',
                                  width=4,
                                  validate="focusout",
                                  textvariable=self.v_latdeg,
                                  validatecommand=self.onLatValDeg)
        self.e_lat_mm = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                 bg='white',
                                 width=3,
                                 validate="focusout",
                                 textvariable=self.v_latmin,
                                 validatecommand=self.onLatValMin)
        self.e_lat_ss = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                 bg='white',
                                 width=3,
                                 validate="focusout",
                                 textvariable=self.v_latsec,
                                 validatecommand=self.onLatValSec)
        self.l_lon = tk.Label(text='Longitude (+/-ddd mm ss): ',
                              font=(FONT, FONTSIZE1, BOLD),
                              justify=tk.LEFT)
        self.e_lon_deg = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                  bg='white',
                                  width=4,
                                  validate="focusout",
                                  textvariable=self.v_londeg,
                                  validatecommand=self.onLonValDeg)
        self.e_lon_mm = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                 bg='white',
                                 width=3,
                                 validate="focusout",
                                 textvariable=self.v_lonmin,
                                 validatecommand=self.onLonValMin)
        self.e_lon_ss = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                                 bg='white',
                                 width=3,
                                 validate="focusout",
                                 textvariable=self.v_lonsec,
                                 validatecommand=self.onLonValSec)
        self.l_elev = tk.Label(text='Elevation (m msl): ',
                               font=(FONT, FONTSIZE1, BOLD),
                               justify=tk.LEFT)
        self.e_elev = tk.Entry(font=(FONT, FONTSIZE1, NORMAL),
                               bg='white',
                               width=6,
                               validate="focusout",
                               textvariable=self.v_elevation,
                               validatecommand=self.onElevVal)
        self.b_insert = tk.Button(self.root,
                                  text="Insert into DB",
                                  width=BUTTONWIDTH,
                                  command=self.db_insert,
                                  bg='lavender')
        self.b_delete = tk.Button(self.root,
                                  text="Delete from DB",
                                  width=BUTTONWIDTH,
                                  command=self.db_delete,
                                  bg='lavender')
        self.b_exit = tk.Button(self.root,
                                text="Exit",
                                width=BUTTONWIDTH,
                                command=self.do_exit,
                                bg='pink')
        self.b_slist = tk.Button(self.root,
                                 text="List Stat.(Pattern)",
                                 width=int(BUTTONWIDTH * 1.5),
                                 command=self.db_list)
        self.e_slist = tk.Entry(self.root,
                                justify=tk.LEFT,
                                bg='white',
                                textvariable=self.v_statpat)
        self.e_slist.bind('<Return>', self.db_list)

        # Tool-Tips
        ToolTip.ToolTip(self.b_slist, text=TIP_STATPATTERN)
        ToolTip.ToolTip(self.e_slist, text=TIP_STATPATTERN)
        ToolTip.ToolTip(self.e_station, text=TIP_STATION)
        ToolTip.ToolTip(self.e_lat_deg, text=TIP_DEGREES)
        ToolTip.ToolTip(self.e_lat_mm, text=TIP_DEGREES)
        ToolTip.ToolTip(self.e_lat_ss, text=TIP_DEGREES)
        ToolTip.ToolTip(self.e_lon_deg, text=TIP_DEGREES)
        ToolTip.ToolTip(self.e_lon_mm, text=TIP_DEGREES)
        ToolTip.ToolTip(self.e_lon_ss, text=TIP_DEGREES)

        # Place the Widgets
        ROW = 0
        l_title.grid(row=ROW, column=0, columnspan=MAXCOLUMN - 1,
                     rowspan=1, sticky='ew')
        self.button_about.grid(row=ROW, column=MAXCOLUMN - 1, sticky='e',
                               padx=5, pady=5)
        ROW += 1
        self.l_database.grid(row=ROW, column=0, columnspan=MAXCOLUMN - 1,
                             sticky='w')
        self.button_help.grid(row=ROW, column=MAXCOLUMN - 1, sticky='e',
                              padx=5, pady=5)
        ROW += 1
        ttk.Separator(orient='horizontal').grid(row=ROW, column=0,
                                                columnspan=MAXCOLUMN,  # + 1,
                                                sticky='ew')
        ROW = ROW + 1
        self.l_station.grid(row=ROW, column=0, sticky='e')
        self.e_station.grid(row=ROW, column=1,
                            columnspan=3, sticky='w')
        ROW = ROW + 1
        self.l_name.grid(row=ROW, column=0, sticky='e')
        self.e_name.grid(row=ROW, column=1,
                         columnspan=MAXCOLUMN - 1,
                         sticky='w')
        ROW = ROW + 1
        self.l_lat.grid(row=ROW, column=0,
                        columnspan=1, sticky='e')
        self.e_lat_deg.grid(row=ROW, column=1,
                            columnspan=1, sticky='ew')
        self.e_lat_mm.grid(row=ROW, column=2,
                           columnspan=1, sticky='ew')
        self.e_lat_ss.grid(row=ROW, column=3,
                           columnspan=1, sticky='ew')
        ROW = ROW + 1
        self.l_lon.grid(row=ROW, column=0,
                        columnspan=1, sticky='e')
        self.e_lon_deg.grid(row=ROW, column=1,
                            columnspan=1, sticky='ew')
        self.e_lon_mm.grid(row=ROW, column=2,
                           columnspan=1, sticky='ew')
        self.e_lon_ss.grid(row=ROW, column=3,
                           columnspan=1, sticky='ew')
        self.b_insert.grid(row=ROW, column=MAXCOLUMN - 1, sticky='e',
                           padx=PADX)
        ROW = ROW + 1
        self.l_elev.grid(row=ROW, column=0,
                         columnspan=1, sticky='e')
        self.e_elev.grid(row=ROW, column=1,
                         columnspan=1, sticky='ew')
        self.b_delete.grid(row=ROW, column=MAXCOLUMN - 1, sticky='e',
                           padx=PADX)
        ROW = ROW + 1
        ttk.Separator(orient='horizontal').grid(row=ROW, column=0,
                                                columnspan=MAXCOLUMN,  # + 1,
                                                sticky='ew',
                                                pady=PADY)
        ROW = ROW + 1
        self.b_slist.grid(row=ROW, column=0, sticky='e',
                          padx=5, pady=5)
        self.b_exit.grid(row=ROW, column=MAXCOLUMN - 1, sticky='e',
                         padx=PADX, pady=PADY)
        self.e_slist.grid(row=ROW, column=1, columns=3, sticky='ew',
                          padx=5, pady=5)
        ROW += 1
        self.scr_txt.grid(
            row=ROW, column=0, columnspan=MAXCOLUMN,
            sticky='ew',
            pady=PADY)

    def about(self):
        tkmb.showinfo('About', ABOUTTXT)
        return

    def help(self):
        tkmb.showinfo('Short Help', TITLE0 + '\n\n' + __doc__)

    def db_delete(self):
        global database
        station = self.v_station.get().upper().strip()
        rmdb = "Delete station '" + station + "' in DB?"
        yn = tkmb.askyesno('Delete station?', rmdb)
        if yn:
            delete_station(database, station)
            log_message(
                self.logfile,
                self.scr_txt,
                'Station ' +
                station +
                ' deleted from database')
        return

    def db_insert(self):
        """insert station into db"""
        global database
        station = self.v_station.get().upper().strip()
        name = self.v_name.get()
        elevation = int(self.v_elevation.get())
        lat_dd = int(self.v_latdeg.get())
        lat_mm = int(self.v_latmin.get())
        lat_ss = int(self.v_latsec.get())
        check_lat = check_ddmmss(lat_dd, lat_mm, lat_ss)
        lon_dd = int(self.v_londeg.get())
        lon_mm = int(self.v_lonmin.get())
        lon_ss = int(self.v_lonsec.get())
        check_lon = check_ddmmss(lon_dd, lon_mm, lon_ss)
        mbox = 'Station : {0:}\n'.format(station)
        mbox = mbox + 'Name   : {0}\n'.format(name)
        mbox = mbox + 'Lat.   : {0:}d {1:}m {2:}s\n'.format(
            lat_dd, lat_mm, lat_ss)
        mbox = mbox + 'Lon.   : {0:}d {1:}m {2:}s\n'.format(
            lon_dd, lon_mm, lon_ss)
        mbox = mbox + 'Elev.  : {0:}'.format(elevation)
        if check_lat and check_lon:
            yn = tkmb.askyesno('Insert into DB',
                               mbox)
            if yn:
                insert_replace_station(database, station, name,
                                       lat_dd, lat_mm, lat_ss,
                                       lon_dd, lon_mm, lon_ss,
                                       elevation)
                log_message(self.logfile, self.scr_txt, '\n' + mbox)
        else:
            erm = 'Error in the common signs of dd mm ss\n\n'
            mbox = erm + mbox
            tkmb.showerror(erm, mbox + '\n\nPlease correct!')
        return

    def db_list(self, event=None):
        allstationsmsg = """You are going to list all stations. This may take a while.
        Are you sure?"""
        pattern = self.v_statpat.get().strip().upper()
        if pattern == '*':
            if not tkmb.askyesno('List all stations?', allstationsmsg):
                return
        slist = read_station_list_by_wildcard(database, pattern)
        message = ""
        nlist = sorted(slist, key=lambda loc: loc[0])
        for ls in nlist:
            name = trunc_str(ls[1], 40)
            latsign = sign_of_dms(ls[2], ls[3], ls[4], plus='N', minus='S')
            lonsign = sign_of_dms(ls[5], ls[6], ls[7], plus='E', minus='W')
            l = (
                "{0:10}  {1:40}   {2:3}°{3:02}'{4:02}\"{5:1}   {6:3}°{7:02}'{8:02}\"{9:1}   {10:5} m".format(
                    ls[0], name, abs(ls[2]), abs(ls[3]), abs(ls[4]), latsign,
                    abs(ls[5]), abs(ls[6]), abs(ls[7]), lonsign, int(ls[8])))
            message = message + '\n' + l
        log_message(self.logfile, self.scr_txt, message)

    def do_exit(self):
        """Exit GUI"""
        OK = tkmb.askokcancel("Exit AdNiteDB?", "Exit really?",
                              default=tkmb.CANCEL)
        if OK:
            self.root.destroy()
        return

    def onIndicValidate(self):
        """Check the location indicator"""
        err_msg = 'Invalid location indicator'

        def IndicError(stat):
            e = stat + '\n' + err_msg
            tkmb.showerror('Error in Location Indicator', e)
            self.e_station.selection_clear()
            self.e_station.focus_set()
            self.e_station.select_range(0, tk.END)

        valid = True
        s = self.v_station.get().strip().upper()
        if not re.search(RESTATION, s):
            valid = False
            err_msg = err_msg + \
                "\nstart with '_', min. 1, max 9 more num-chars: "
            IndicError(s)
        return valid

    def onLatValDeg(self):
        deg = self.v_latdeg.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -90) or (dd > +90):
                valid = False
        except:
            valid = False
        if valid:
            pass
        else:
            log_message(self.logfile, self.scr_txt,
                        'ERROR - Invalid Latitude degree: ' + deg)
            self.e_lat_deg.selection_clear()
            self.e_lat_deg.focus_set()
            self.e_lat_deg.select_range(0, tk.END)
        self.v_latdeg.set(deg)
        return valid

    def onLatValMin(self):
        # print('onLatValMin')
        deg = self.v_latmin.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -59) or (dd > 59):
                valid = False
        except:
            valid = False
        if valid:
            pass  # print('Valid Latitude Minute: ', dd)
        else:
            log_message(self.logfile, self.scr_txt,
                        'ERROR - Invalid Latitude Minute: ' + deg)
            self.e_lat_mm.selection_clear()
            self.e_lat_mm.focus_set()
            self.e_lat_mm.select_range(0, tk.END)
        self.v_latmin.set(deg)
        return valid

    def onLatValSec(self):
        deg = self.v_latsec.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -59) or (dd > 59):
                valid = False
        except:
            valid = False
        if valid:
            pass
        else:
            log_message(self.logfile, self.scr_txt,
                        'ERROR - Invalid Latitude Seconds: ' + deg)
            self.e_lat_ss.selection_clear()
            self.e_lat_ss.focus_set()
            self.e_lat_ss.select_range(0, tk.END)
        self.v_latsec.set(deg)
        return valid

    def onLonValDeg(self):
        deg = self.v_londeg.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -180) or (dd > 180):
                valid = False
        except:
            valid = False
        if valid:
            pass
        else:
            log_message(self.logfile, self.scr_txt,
                        'ERROR - Invalid Longitude degree: ' + deg)
            # self.e_lat_deg.selection_clear()
            # self.e_lat_deg.focus_set()
            # self.e_lat_deg.select_range(0, tk.END)
            self.e_lon_deg.selection_clear()
            self.e_lon_deg.focus_set()
            self.e_lon_deg.select_range(0, tk.END)
        self.v_londeg.set(deg)
        return valid

    def onLonValMin(self):
        # print('onLonValMin')
        deg = self.v_lonmin.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -59) or (dd > 59):
                valid = False
        except:
            valid = False
        if valid:
            pass  # print('Valid Longitude Minute: ', dd)
        else:
            log_message(self.logfile, self.scr_txt,
                        'ERROR - Invalid Longitude Minute: ' + deg)
            self.e_lon_mm.selection_clear()
            self.e_lon_mm.focus_set()
            self.e_lon_mm.select_range(0, tk.END)
        self.v_lonmin.set(deg)
        return valid

    def onLonValSec(self):
        deg = self.v_lonsec.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -59) or (dd > 59):
                valid = False
        except:
            valid = False
        if valid:
            pass
        else:
            log_message(
                self.logfile,
                self.scr_txt,
                'ERROR - Invalid Longitude Seconds: ' +
                deg)
            self.e_lon_ss.selection_clear()
            self.e_lon_ss.focus_set()
            self.e_lon_ss.select_range(0, tk.END)
        self.v_lonsec.set(deg)
        return valid

    def onElevVal(self):
        # print('onElevVal')
        deg = self.v_elevation.get()
        if deg.strip() == "":
            deg = '0'
        try:
            dd = int(deg)
            valid = True
            if (dd < -1000) or (dd >= 9000):
                valid = False
        except:
            valid = False
        if valid:
            pass  # print('Valid Elevation: ', dd)
        else:
            log_message(
                self.logfile,
                self.scr_txt,
                'ERROR - Invalid Elevation: ' +
                deg)
            self.e_elev.selection_clear()
            self.e_elev.focus_set()
            self.e_elev.select_range(0, tk.END)
        self.v_elevation.set(deg)
        return valid


def main():
    global database
    # Argument parsing
    adnitehelp = """Environment: adnite2' or 'adnite3',
    defaults to 'adnite2', defines the directory and name for the station
    database."""
    parser = argparse.ArgumentParser()
    parser.add_argument("-a", "--adnite", default='adnite2',
                        help=adnitehelp)
    parser.add_argument("-d", "--debug", action='store_true',
                        help="Switch on output to terminal")
    args = parser.parse_args()
    # Debug on/off, off by default
    if not args.debug:
        f = open(os.devnull, 'w')
        sys.stdout = f
    # Database guessing
    database = ''
    if args.adnite == 'adnite2':
        database = op.expanduser('~/.adnite2/stations.sqlite')
    elif args.adnite == 'adnite3':
        database = op.expanduser('~/.adnite3/stations.sqlite')
    # Check if the data base exits, otherwise choose one or exit
    if not op.exists(database):
        r0 = tk.Tk()  # Do get rid of the tiny unnecessary root window
        r0.withdraw()  # hide it
        msg1 = 'No SqLite-Station DB found - Select one or cancel'
        fn = tkfd.askopenfilename(defaultextension='sqlite',
                                  filetypes=[('SqLite3', '*.sqlite')],
                                  title=msg1)
        r0.deiconify()  # show it, otherwise it cannot be destroyed(!?!)
        r0.destroy()  # destroy it
        if fn:
            database = fn
        else:
            msgbye = 'No SqLite3 station database selected. \n' + \
                "EXIT program '" + __progname__ + "'"
            tkmb.showerror('Bye bye AdNiteDB', msgbye)
            sys.exit(msgbye)

    # Display GUI
    root = tk.Tk(className=TITLE0)
    AdniteDB(root)
    root.mainloop()


if __name__ == '__main__':
    main()
