# Created on 2016-03-15 by D.Th.
"""Contains meteorological and astrop-hysical routines"""

from numpy import deg2rad, rad2deg
import datetime
import ephem

import adnite2.constants as constants
import adnite2.dattims as dattims
import adnite2.inout as inout
import adnite2.version as version
import numpy as np


CLSKYALBEDO = constants.ALBEDO_ATMOS['NRM']  # : clear sky atmos. albedo
# CLSKYALBEDO = constants.ALBEDO_ATMOS['RAL']  #: clear sky atmos. albedo
# CLSKYALBEDO = constants.ALBEDO_ATMOS['TRB']  #: clear sky atmos. albedo
MAXLOUDN = constants.CLOUDAMOUNT_MAX


def py_localnoon_midnight(longitude=0.0,
                          pytimeutc=datetime.datetime.utcnow()):
    """Calculates local mean noon and midnight in python datettime in UTC
    given longitude and python datetime"""
    pydate = pytimeutc.date()
    dd = ephem.Date(ephem.Date(pydate) + 0.5)
    dt = float(longitude) / 15.0 / 24.
    ddnoon = ephem.Date(dd - dt)
    ddmidnight = ephem.Date(ddnoon + 0.5)
    return (dattims.ephemdatetime2pydatetime(ddnoon),
            dattims.ephemdatetime2pydatetime(ddmidnight))


def sunriseset(location, pydatetime, horizon='-0:50'):
    """Calculates sun rise and set for a *location* and around the
    *time* above the *horizon*. Time should be the mean local noon
    or at least a time where the sun is above the horizon. It Returns
    a dictionary with {'prevsunset':set_prev, 'prevsunrise':rise_prev,
    'nextsunset':set_next,'nextsunrise':rise_next}

    Returned date-times are pyephem objects
    """
    NVRUP = dattims.MINDATTIM
    NVRDOWN = dattims.MAXDATTIM
    loc = ephem.Observer()
    loc.horizon = str(horizon)  # ####### str neu???
    # print "sunriseset-horizon", horizon
    loc.pressure = 0.0  # refraction is accounted for in horizon
    loc.long = str(location['lon'])  # ###########################
    loc.lat = str(location['lat'])
    loc.elevation = float(location['elev'])
    loc.date = pydatetime
    sun = ephem.Sun()  # @UndefinedVariable
    try:
        set_prev = loc.previous_setting(sun, use_center=True)
        set_next = loc.next_setting(sun, use_center=True)
        rise_prev = loc.previous_rising(sun, use_center=True)
        rise_next = loc.next_rising(sun, use_center=True)
    except ephem.NeverUpError:
        set_prev, rise_prev, set_next, rise_next = NVRUP, NVRUP, NVRUP, NVRUP
    except ephem.AlwaysUpError:
        set_prev, rise_prev, set_next, rise_next = NVRDOWN, NVRDOWN, \
            NVRDOWN, NVRDOWN
    sunsrdict = {'prevsunset': set_prev, 'prevsunrise': rise_prev,
                 'nextsunset': set_next, 'nextsunrise': rise_next}
    return sunsrdict


def moonriseset(location, pydatetime, horizon='-0:34'):
    """Calculates moon rise and set for a *location* and around the
    time *pydatetime* above the *horizon*. Standard horizon accounts for
    atmospheric refraction.

    **For reasonable results the time must be chosen
    carefully.** If the moon has e.g. allready set, the next set will be
    tommorow though he previous rise has been today. So not all events
    of the day and the night might be caught.

    The function returns a dictionary with the results.
    The results can be derived by:

    * dict['prevrise]
    * dict['prevset']
    * dict['nextrise']
    * dict['nextset']

    Returned date-times are pyephem objects
    """
    NVRUP = dattims.MINDATTIM
    NVRDOWN = dattims.MAXDATTIM
    loc = ephem.Observer()
    loc.horizon = horizon
    loc.pressure = 0.0  # no calculated refraction (only fixed by)
    loc.lon = str(location['lon'])
    loc.lat = str(location['lat'])
    loc.elevation = float(location['elev'])
    loc.date = pydatetime
    moon = ephem.Moon()
    try:
        set_prev = loc.previous_setting(moon)
        set_next = loc.next_setting(moon)
        rise_prev = loc.previous_rising(moon)
        rise_next = loc.next_rising(moon)
    except ephem.NeverUpError:
        set_prev, rise_prev, set_next, rise_next = NVRUP, NVRUP, NVRUP, NVRUP
    except ephem.AlwaysUpError:
        set_prev, rise_prev, set_next, rise_next = NVRDOWN, NVRDOWN, \
            NVRDOWN, NVRDOWN
    moonsrdict = {'prevset': set_prev, 'prevrise': rise_prev,
                  'nextset': set_next, 'nextrise': rise_next}
    return moonsrdict


def moonphase(pydatetime):
    """ Phase of the moon in percent of the apparent area of the full moon"""
    m = ephem.Moon()
    m.compute(pydatetime)
    return m.phase


def moonfullnew(pydatetime):
    """Calculates previous and next main moon phases (new, 1.quater, full,
    last quater). Returnes a dictionary {'prev_new_moon':pnm,
    'next_new_moon':nnm, 'prev_1st_quarter':pfq,
    'next_1st_quarter':nfq, 'prev_full_moon':pfm,
    'next_full_moon':nfm, 'prev_last_quarter':plq,
    'next_last_quarter':nlq}
    """
    pnm = ephem.previous_new_moon(pydatetime)
    nnm = ephem.next_new_moon(pydatetime)
    pfq = ephem.previous_first_quarter_moon(pydatetime)
    nfq = ephem.next_first_quarter_moon(pydatetime)
    pfm = ephem.previous_full_moon(pydatetime)
    nfm = ephem.next_full_moon(pydatetime)
    plq = ephem.previous_last_quarter_moon(pydatetime)
    nlq = ephem.next_last_quarter_moon(pydatetime)
    mphasedict = {'prev_new_moon': pnm, 'next_new_moon': nnm,
                  'prev_1st_quarter': pfq, 'next_1st_quarter': nfq,
                  'prev_full_moon': pfm, 'next_full_moon': nfm,
                  'prev_last_quarter': plq, 'next_last_quarter': nlq}
    return mphasedict


def calc_astro_night(location, datetimeutc):
    """Takes

    * location dictionary with the geographic data of the location as
      returned by stationlistdb
    * python dattime style utc time

    and returns a tuple with all usefull astronomic standard data
    structures of the midnight following the noon of the day:

    * local noon
    * local midnight 12 hors after noon
    * sun rise and set
    * civil twilight
    * nautical twilight
    * astronomical twilight
    * moon rise and set
    * moon phase in percent
    * moon full and new
    * next full moon
    * next new moon
    """
    # lon = deg2rad(float(location['lon']))
    lon = float(location['lon'])
    locnoon, locmidnight = py_localnoon_midnight(lon, datetimeutc)
    srs = sunriseset(location, locmidnight, constants.SUNRISESET)
    ctw = sunriseset(location, locmidnight, constants.TWLCIVIL)
    ntw = sunriseset(location, locmidnight, constants.TWLNAUTIC)
    atw = sunriseset(location, locmidnight, constants.TWLASTRO)
    mrs = moonriseset(location, locmidnight)
    mphase = moonphase(locmidnight)
    mfn = moonfullnew(locmidnight)
    nfm = mfn['next_full_moon']
    nnm = mfn['next_new_moon']
    nightdict = {'localnoon': locnoon,
                 'localmidnight': locmidnight,
                 'sunriseset': srs,
                 'civiltwilight': ctw,
                 'nauticaltwilight': ntw,
                 'astronomicaltwilight': atw,
                 'moonriseset': mrs,
                 'moonphase': mphase,
                 'moonfullnew': mfn,
                 'nextfullmoon': nfm,
                 'nextnewmoon': nnm}
    return nightdict


def extract_relevant_astro_night(nightdict):
    """Extracts the relevant astronomical data from
    dictionary nightdict (result of 'calc_astro_night'
    """
    msr = nightdict['moonriseset']
    if msr['prevset'] > msr['prevrise']:
        prevm = ('s', dattims.ephemdatetime2pydatetime(msr['prevset']))
        # print('Prevmoon', prevm)
    else:
        prevm = ('r', dattims.ephemdatetime2pydatetime(msr['prevrise']))
    if msr['nextset'] < msr['nextrise']:
        nextm = ('s', dattims.ephemdatetime2pydatetime(msr['nextset']))
    else:
        nextm = ('r', dattims.ephemdatetime2pydatetime(msr['nextrise']))
    relevantdict = {
        'localnoon': nightdict['localnoon'],
        'nextfullmoon':
        dattims.ephemdatetime2pydatetime(
            nightdict['nextfullmoon']),
        'nextnewmoon':
        dattims.ephemdatetime2pydatetime(nightdict['nextnewmoon']),
        'sunset':
        dattims.ephemdatetime2pydatetime(
            nightdict['sunriseset']['prevsunset']),
        'sunrise':
        dattims.ephemdatetime2pydatetime(
            nightdict['sunriseset']['nextsunrise']),
        'ecet':
        dattims.ephemdatetime2pydatetime(
            nightdict['civiltwilight']['prevsunset']),
        'bcmt':
        dattims.ephemdatetime2pydatetime(
            nightdict['civiltwilight']['nextsunrise']),
        'enet':
        dattims.ephemdatetime2pydatetime(
            nightdict['nauticaltwilight']['prevsunset']),
        'bnmt':
        dattims.ephemdatetime2pydatetime(
            nightdict['nauticaltwilight']['nextsunrise']),
        'moonphase': nightdict['moonphase'],
        'nextmoonevent': nextm,
        'prevmoonevent': prevm
    }
    return relevantdict


def mag2lux(mag):
    """Converts astronomical magnitude to illuminace (Beleuchtungsstaerke)
    in lux normal to the direction of the object according to the
    formula given at
    http://members.ziggo.nl/jhm.vangastel/Astronomy/Formules.pdf
    """
    lux = 10.0 ** ((-14.18 - mag) / 2.5)
    # print('mag2lux - Mag:', mag)
    # print('mag2lux - Lux:', lux)
    return lux


def relatmass(altrad):
    """ Relative atmospheric mass according to Pickering(2002) as
    a function of altitude in rad."""
    altdeg = rad2deg(altrad)
    if altdeg < 0.0:
        altdeg = 0.0
    q = altdeg + 244.0 / (165.0 + 57.0 * np.power(altdeg, 1.1))
    M = 1.0 / np.sin(deg2rad(q))
    return M


def noskyilluminance(altrad, illum0):
    """Calculates the illuminance on an horizontal surface without
    extinction (no atmosphere!). Does not take (numpy-)arrays!

    *Input:*

    * altrad ... altitude of the celestial body in rad(!)

    * illum0 ... illuminance in appropriate units of the celestial body
      normal to the beam witout atmospheric attenuation (e.g. output of
      function 'mag2lux'

    *Output*

    * illuminance ... at a horizontal surface in whatever units illum0 has
      been passed (with no extra atmospheric attenuation)
    """
    if altrad < 0.0:
        illum_noatm_proj = 0.
    else:
        illum_noatm_proj = illum0 * np.sin(altrad)
    return illum_noatm_proj


def turbidskyilluminance(altrad, illum0):
    """Calculates the illuminace by the direct beam projected to the
    horizontal surface and with atmosphereic extinction."""
    if altrad < 0.0:
        ill1 = 0.0
    else:
        ill1 = noskyilluminance(altrad, illum0)
    M = relatmass(altrad)
    ill = ill1 * np.power(constants.Q_TRANSPARENCY, M)
    # print('turbidskyill - illum0: ', illum0)
    # print('turbidskyill - ill: ', ill)
    return ill


CLOUDALBEDOS = {'NS': 0.9, 'SC': 0.7, 'ST': 0.65}


def illuminance_moon_ESAA2013(altitude, elongation):
    """Ground illuminances for the moon according to
    <Explanatory Suppl. of the Astron.Almanac>, 3rd ed.,
    Urban,Seidelmann (2013), providing a clear atmosphere and
    no clouds .

    * altitude ... array of altitudes in rad
    * elongation ... array of sun to moon elongation

    Returns array of clear sky moon illuminances in lux"""
    H2 = 20.0  # upper approx. thresholds in deg
    H1 = 5.0  # medium
    H0 = -5.  # lower validity range is -0.8, to make it smooth -5.0!
    # initializing
    H = np.rad2deg(altitude)
    X = H / 90.
    F = 180. - np.abs(np.rad2deg(elongation))
    log10m = np.empty_like(H)
    L1 = np.empty_like(H)
    L2 = np.empty_like(H)
    L3 = np.empty_like(H)
    I0 = np.zeros(len(H))
    l0 = np.zeros(len(H))
    l1 = np.zeros(len(H))
    l2 = np.zeros(len(H))
    l3 = np.zeros(len(H))
    # coefficients for different moon altitudes
    k = np.less(H, H0)
    l0[k] = None
    l1[k] = None
    l2[k] = None
    l3[k] = None
    k = np.greater_equal(H, H0)
    l0[k] = -2.79
    l1[k] = 24.27
    l2[k] = -252.95
    l3[k] = 1321.29
    k = np.greater_equal(H, H1)
    l0[k] = -2.58
    l1[k] = 12.58
    l2[k] = -42.58
    l3[k] = 59.06
    k = np.greater_equal(H, H2)
    l0[k] = -1.95
    l1[k] = 4.06
    l2[k] = -4.24
    l3[k] = 1.56
    # above horizon
    k = np.greater_equal(H, H0)
    L1[k] = (l0[k] + l1[k] * X[k] +
             l2[k] * (X[k] * X[k]) +
             l3[k] * (X[k] * X[k] * X[k]))
    L2[k] = (-8.68e-3 * F[k] -
             2.2e-9 * F[k] * F[k] * F[k] * F[k])
    L3[k] = 0.  # 2 * np.log10(pp / 0.951)   # pi ... paralaxe neclected
    log10m[k] = L1[k] + L2[k]  # + L3[k]
    I0[k] = np.power(10., log10m[k])
    # well below horizon
    k = np.less(H, H0)
    I0[k] = 0.0
    return I0


def illuminance_below_clouds2(illum_nosky,
                              cloudiness=0.,
                              albedo_surface=0.2,
                              albedo_cloud_top=0.7,
                              albedo_cloud_bottom=None,
                              albedo_atmosphere_top=0.14,
                              albedo_atmosphere_bottom=None):
    """ Downward directed illumination S below a cloud cover as a function of:
      (WITH ATMOSPHERIC SCATTERING!)

      * illum_nosky, I_0 .. illumination above the cloud cover
      * cloudiness in tenth (0..10)
      * albedo_surface (0..1), A_s
      * albedo_cloud_top (0..1), A_t
      * albedo_cloud_bottom (0..1), A_b
      * albedo_atmosphere_top (0..1), d_t
      * albedo_atmosphere_bottom (0..1), d_b:

    According to `D. Thaler (2015)
    <http://www.foehnwall.at/multiplescattering1.html>`_:

    .. math::

       \\boxed{S/I_0 = \\frac{(1-d_t) - N(A_t - d_t)}{1-d_b A_s) -
       N A_s (A_b - d_b)}}

        """
    if albedo_cloud_bottom is None:
        albedo_cloud_bottom = albedo_cloud_top
    if albedo_atmosphere_bottom is None:
        albedo_atmosphere_bottom = albedo_atmosphere_top

    I0 = illum_nosky
    N = cloudiness / MAXLOUDN
    As = albedo_surface
    At = albedo_cloud_top
    Ab = albedo_cloud_bottom
    dt = albedo_atmosphere_top
    db = albedo_atmosphere_bottom
    S = I0 * ((1 - dt) - N * (At - dt)) / \
        ((1 - As * db) - N * As * (Ab - db))
    return S


def twilight_illuminance(hr):
    """
    Estimates clear sky twilight illuminances on the ground dependent on
    (negativ) sun altitude. A exponetial-linear approach is used:

    .. math::

       \\boxed{S(h) = S_0 + (S_0 - S_{-\infty}) e^{b h}}

       b = \ln{(S_H - S_{-\infty})/(S_0 - S_{-\infty})}

    with :math:`h` the sun altitude, :math:`S_0 = S(h=0)`,
    :math:`S_H = S(h=H)`, e.g. :math:`H = -6 \; \mathrm{deg}`
    (ECET) as extra value, and :math:`S_{-\infty}` as the deep night
    background value (star light plus air glow).

    Values from:
    SEAN E. URBAN , P. KENNETH SEIDELMANN:
    Explanatory Supplement to the Astronomical Almanac, 2012
    """
    S0 = constants.SUNSET_ILLUMINATION
    SMINF = constants.BACKGROUND_ILLUMINATION
    H = constants.HCIVIL  # : deg, reference sun height (ECET)
    SH = constants.TWLCIVIL_ILLUMINATION  # : lux, sun alt = H deg
    hd = np.rad2deg(hr)
    dS0 = (S0 - SMINF)
    dSH = (SH - SMINF)
    B = np.log(dSH / dS0) / H
    S = dS0 * np.exp(B * hd) + SMINF
    return S


def sundata(location, pydatetimearray):
    """Calculates sun altitude, azimuth, size and magnitude for a given
    location between to dates with a given time increment.

    *Parameters*:

    * location ... dictionary with the geographic data of the location
      as returned by e.g. stationlistdb
    * pydatetimearray ... a list with standard python datetime-objects
    * configdict ... dictionary with configuration data (constants, etc.)
      (previously read from file)

    *Returns*: a dictionary with:

    * 'ephemdatetime' ... numpy array with pyephem.Dates-Data
    * 'altitude' ... numpy array with altitudes in pyephem.degrees -
      data
    * 'azimuth' ... numpy array with azimuth in pyephem.degrees - data
    * 'size' ...  numpy array with azimuth in pyephem.size - data
    * 'magnitude' ... numpy array with azimuth in pyephem.mag - data
    """
    # convert python datetimes to pyephem datetimes
    ephemdattim = dattims.pydatetime2ephemdatetime_array(pydatetimearray)
    ephemdattim = np.array(ephemdattim)
    #
    altitude = np.empty_like(ephemdattim)
    azimuth = np.empty_like(ephemdattim)
    size = np.empty_like(ephemdattim)
    magnitude = np.empty_like(ephemdattim)
    illuminance0 = np.empty_like(ephemdattim)
    illuminance_nosky = np.empty_like(ephemdattim)
    illuminance_turbidsky = np.empty_like(ephemdattim)

    ort = ephem.Observer()
    ort.lat = str(location['lat'])
    ort.lon = str(location['lon'])
    ort.elevation = float(location['elev'])
    ort.pressure = 0.0  # *NECESSARY* to avoid strange refraction effects
    sun = ephem.Sun(ort)  # @UndefinedVariable
    ind = 0
    for d in ephemdattim:
        ort.date = d
        sun.compute(ort)
        altitude[ind] = sun.alt
        azimuth[ind] = sun.az
        size[ind] = sun.size
        magnitude[ind] = sun.mag
        illuminance0[ind] = mag2lux(sun.mag)
        illuminance_nosky[ind] = noskyilluminance(sun.alt,
                                                  illuminance0[ind])
        illuminance_turbidsky[ind] = turbidskyilluminance(sun.alt,
                                                          illuminance0[ind])
        ind = ind + 1
    sundatadic = {'ephemdatetime': ephemdattim,
                  'altitude': altitude,
                  'azimuth': azimuth,
                  'size': size,
                  'magnitude': magnitude,
                  'illuminance0': illuminance0,
                  'illuminance_nosky': illuminance_nosky,
                  'illuminance_turbidsky': illuminance_turbidsky
                  }
    return sundatadic


def moondata(location, pydatetimearray):
    """Calculates moon altitude, azimuth, size and magnitude for a given
    location between to dates with a given time increment.

    *Parameters*:

    * location ... dictionary with the geographic data of the location
      as returned by e.g. stationlistdb
    * pydatetimearray ... a list with standard python datetime-objects

    *Returns* a dictionary with:

    * 'pydatetimes': the same list with standard python datetime-objects as
      input
    * 'ephemdatetime' ... numpy array with pyephem.Dates-Data
    * 'altitude' ... numpy array with altitudes in pyephem.degrees -
      data (radiant)!
    * 'azimuth' ... numpy array with azimuth in pyephem.degrees - data
      (radiant!)
    * 'elongation' ... geocentric angle to sun (radiant!)
    * 'size' ...  numpy array with azimuth in pyephem.size - data
    * 'magnitude' ... numpy array with azimuth in pyephem.mag - data
    * Different sorts of illuminances:
    """
    # convert python datetimes to pyephem datetimes
    # ILLUM_FULLMOON_MEAN = float(configdict['ILLUM_FULLMOON_MEAN'])
    ephemdattim = dattims.pydatetime2ephemdatetime_array(pydatetimearray)
    ephemdattim = np.array(ephemdattim)
    altitude = np.empty_like(ephemdattim)
    azimuth = np.empty_like(ephemdattim)
    elongation = np.empty_like(ephemdattim)
    size = np.empty_like(ephemdattim)
    magnitude = np.empty_like(ephemdattim)
    illuminance0 = np.empty_like(ephemdattim)
    illuminance_nosky = np.empty_like(ephemdattim)
    illuminance_turbidsky = np.empty_like(ephemdattim)
    rel_illuminance0 = np.empty_like(ephemdattim)
    rel_illuminance = np.empty_like(ephemdattim)

    phase = np.empty_like(ephemdattim)
    ort = ephem.Observer()
    ort.lat = str(location['lat'])
    ort.lon = str(location['lon'])
    ort.elevation = float(location['elev'])
    ort.pressure = 0.0  # *NECESSARY* to avoid strange refraction effects
    moon = ephem.Moon(ort)
    ind = 0
    for d in ephemdattim:
        ort.date = d
        moon.compute(ort)
        altitude[ind] = moon.alt
        azimuth[ind] = moon.az
        elongation[ind] = moon.elong
        size[ind] = moon.size
        magnitude[ind] = moon.mag

        illuminance0[ind] = mag2lux(moon.mag)
        illuminance_nosky[ind] = noskyilluminance(moon.alt,
                                                  illuminance0[ind])
        illuminance_turbidsky[ind] = turbidskyilluminance(moon.alt,
                                                          illuminance0[ind])
        rel_illuminance0[ind] = \
            illuminance_nosky[ind] / constants.ILLUM_FULLMOON_MEAN
        rel_illuminance[ind] = \
            illuminance_turbidsky[ind] / constants.ILLUM_FULLMOON_MEAN
        phase[ind] = moon.phase
        ind = ind + 1
    illuminance_ESAA2013 = illuminance_moon_ESAA2013(
        altitude, elongation)
    rel_illuminance_ESSAA2013 = illuminance_ESAA2013 / \
        constants.ILLUM_FULLMOON_MEAN
    moondatadic = {'pydatetimes': pydatetimearray,
                   'ephemdatetime': ephemdattim,
                   'altitude': altitude,
                   'azimuth': azimuth,
                   'elongation': elongation,
                   'size': size,
                   'magnitude': magnitude,
                   'illuminance0': illuminance0,
                   'illuminance_nosky': illuminance_nosky,
                   'illuminance_turbidsky': illuminance_turbidsky,
                   'illuminance_ESAA2013': illuminance_ESAA2013,
                   'rel_illuminance0': rel_illuminance0,
                   'rel_illuminance': rel_illuminance,
                   'rel_illuminance_ESAA2013': rel_illuminance_ESSAA2013,
                   'phase': phase}
    return moondatadic


def civilmoonphase(moondata):
    """Defines the moonphases in a very simple way.
    phase 00..20%   0, new
    phase 20..80%   1, 1st qu.
    phase 80-100%   2, full
    phase 20-80%    3, 3st qu."""
    phases = moondata['phase']
    meanphase = (phases[0] + phases[-1]) / 2
    if meanphase < 20:
        cph = 0
    elif meanphase > 80:
        cph = 2
    elif meanphase < phases[-1]:  # waxing, 1st qu.
        cph = 1
    else:  # 3. qu.
        cph = 3
    return cph


def civilmoonphase_symbol(stationdata, moondata):
    """Define moon markersymbol according to moonphase and latitude"""
    cvmph = civilmoonphase(moondata)
    if float(stationdata['lat']) > 0:  # northern hemisphere
        if cvmph == 0:
            fillstyle = 'none'
        elif cvmph == 1:
            fillstyle = 'right'
        elif cvmph == 2:
            fillstyle = 'full'
        else:
            fillstyle = 'left'
    else:  # southern hemisphere
        if cvmph == 0:
            fillstyle = 'none'
        elif cvmph == 1:
            fillstyle = 'left'
        elif cvmph == 2:
            fillstyle = 'full'
        else:
            fillstyle = 'right'
    return fillstyle


def ground2albedo(grounds):
    """Converts ground type array to albedo array"""
    albedos = []
    for gr in grounds:
        Gr = gr.upper()
        albedos.append(constants.ALBEDO_GROUND[Gr])
    return np.array(albedos)


def clouddensity2albedo(clouddensity):
    """Converts cloud density array to albedo array"""
    albedos = []
    for d in clouddensity:
        ca = constants.ALBEDO_CLOUD[d]
        albedos.append(ca)
    return np.array(albedos)


def illuminance_core(pydatetimes,
                     sundata, moondata,
                     cloudoktas, cloudalbedo, groundalbedo):
    """Core of the illuminance calculations (work horse). Accepts not only
    coarse arrays but also bigger fine resolution arrays.

    * pydatetimes ... python datet-time array
    * sundata ... dictionary of arrays with sun data
    * moondata ... dictionary of array with moon data
    * cloudoktas ... array with cloudiness in oktas 0 .. 8
    * cloudalbedo ... array with cloud albedo 0..1
    * groundalbedo ... array with ground albedo 0..1

    All (1-dimensional) arrays must have the same array-length"""

    sund = sundata
    moond = moondata
    groundalb = groundalbedo
    cloudalb = cloudalbedo
    cloudsn = cloudoktas

    sunalt = sund['altitude']
    sunill0 = sund['illuminance_nosky']
    # moonill0 = moond['illuminance_nosky']
    moonill0 = moond['illuminance_ESAA2013']

    sunill = np.empty_like(sunill0)
    moonill = np.empty_like(moonill0)
    twilight0 = twilight_illuminance(sunalt)
    twilight = np.empty_like(moonill0)

    for kk in range(len(sunill0)):
        sunill[kk] = illuminance_below_clouds2(
            sunill0[kk],
            cloudiness=float(
                cloudsn[kk]),
            albedo_surface=groundalb[kk],
            albedo_cloud_top=cloudalb[kk],
            albedo_atmosphere_top=CLSKYALBEDO)
        moonill[kk] = illuminance_below_clouds2(
            moonill0[kk],
            cloudiness=float(
                cloudsn[kk]),
            albedo_surface=groundalb[kk],
            albedo_cloud_top=cloudalb[kk],
            albedo_atmosphere_top=CLSKYALBEDO)
        twilight[kk] = illuminance_below_clouds2(
            twilight0[kk],
            cloudiness=float(
                cloudsn[kk]),
            albedo_surface=groundalb[kk],
            albedo_cloud_top=cloudalb[kk],
            albedo_atmosphere_top=CLSKYALBEDO)
    illuminancecore = {
        'sunilluminance_nosky': sunill0,
        'sunilluminance': sunill,
        'moonilluminance_nosky': moonill0,
        'moonilluminance': moonill,
        'twilightilluminance0': twilight0,
        'twilightilluminace': twilight,
        'totalilluminance0': sunill0 + moonill0 + twilight0,
        'totalilluminance': sunill + moonill + twilight
    }
    return illuminancecore


def illuminancedata_coarse(stationdata, pydatetimes, params):
    """Calulates illuminances dependant on environment data
    for coarse table output

    * stationdata ... stationdatabase record for retrieved station
    * pydatetimes ... (coarse) python datet-time array
    * params ... (coarse) parameters as read from inputfile

    Returns:

    dictionary with usefull data (see source code)"""
    sund = sundata(stationdata, pydatetimes)
    moond = moondata(stationdata, pydatetimes)

    cloudsn = params['cloudsn']
    cloudsnx = params['cloudsnx']
    cloudalb = clouddensity2albedo(params['cloudsd'])
    cloudalbx = clouddensity2albedo(params['cloudsdx'])
    groundalb = ground2albedo(params['ground'])
    groundalbx = ground2albedo(params['groundx'])

    physparams = {'cloudalb': cloudalb,
                  'cloudalbx': cloudalbx,
                  'groundalb': groundalb,
                  'groundalbx': groundalbx}
    inout.print_parameters(physparams, 'Albedos')

    illuminance = illuminance_core(pydatetimes, sund, moond,
                                   cloudsn, cloudalb, groundalb)

    illuminancex = illuminance_core(pydatetimes, sund, moond,
                                    cloudsnx, cloudalbx, groundalbx)

    illuminancedata = {
        'pydatetimes': pydatetimes,
        'ephemdatetime': sund['ephemdatetime'],
        'sunaltitude': sund['altitude'],
        'sunazimuth': sund['azimuth'],
        'sunilluminance_nosky': illuminance['sunilluminance_nosky'],
        'sunilluminance': illuminance['sunilluminance'],
        'sunilluminancex': illuminancex['sunilluminance'],
        'moonaltitude': moond['altitude'],
        'moonazimuth': moond['azimuth'],
        'moonilluminance_nosky': illuminance['moonilluminance_nosky'],
        'moonilluminance': illuminance['moonilluminance'],
        'moonilluminancex': illuminancex['moonilluminance'],
        'twilightilluminance0': illuminance['twilightilluminance0'],
        'twilightilluminace': illuminance['twilightilluminace'],
        'twilightilluminancex': illuminancex['twilightilluminace'],
        'totalilluminance0': illuminance['totalilluminance0'],
        'totalilluminance': illuminance['totalilluminance'],
        'totalilluminancex': illuminancex['totalilluminance']
    }
    return illuminancedata


def illuminancedata_fine(stationdata, pydatetimes, params, timestep):
    """Calulates illuminances dependant on environment data
    for coarse table output

    * stationdata ... stationdatabase record for retrieved station
    * pydatetimes ... (coarse) python datet-time array
    * params ... (coarse) parameters as read from inputfile
    * timestep ... timestep in minutes (as integer), must divide the timespan
      without rest.

    Returns:

    dictionary with usefull fine data arrays (see source code)"""

    pydatetimes_fine = dattims.fine_pydatetime_array(
        pydatetimes,
        d_minutes=constants.TIMESTEP4GRAPHS)

    sund = sundata(stationdata, pydatetimes_fine)
    moond = moondata(stationdata, pydatetimes_fine)

    cldn_coarse = params['cloudsn']
    cldnx_coarse = params['cloudsnx']
    cldalb_coarse = clouddensity2albedo(params['cloudsd'])
    cldalbx_coarse = clouddensity2albedo(params['cloudsdx'])
    grdalb_coarse = ground2albedo(params['ground'])
    grdalbx_coarse = ground2albedo(params['groundx'])

    cldn_fine = inout.fine_reals(pydatetimes, cldn_coarse,
                                 pydatetimes_fine)
    cldnx_fine = inout.fine_reals(pydatetimes, cldnx_coarse,
                                  pydatetimes_fine)
    cldalb_fine = inout.fine_reals(pydatetimes, cldalb_coarse,
                                   pydatetimes_fine)
    cldalbx_fine = inout.fine_reals(pydatetimes, cldalbx_coarse,
                                    pydatetimes_fine)
    grdalb_fine = inout.fine_reals(pydatetimes, grdalb_coarse,
                                   pydatetimes_fine)
    grdalbx_fine = inout.fine_reals(pydatetimes, grdalbx_coarse,
                                    pydatetimes_fine)

    illuminance = illuminance_core(pydatetimes_fine, sund, moond,
                                   cldn_fine, cldalb_fine, grdalb_fine)
    illuminancex = illuminance_core(pydatetimes_fine, sund, moond,
                                    cldnx_fine, cldalbx_fine, grdalbx_fine)

    illuminancedata = {
        'pydatetimes': pydatetimes_fine,
        'ephemdatetime': sund['ephemdatetime'],
        'sunaltitude': sund['altitude'],
        'sunazimuth': sund['azimuth'],
        'sunilluminance_nosky': illuminance['sunilluminance_nosky'],
        'sunilluminance': illuminance['sunilluminance'],
        'sunilluminancex': illuminancex['sunilluminance'],
        'moonaltitude': moond['altitude'],
        'moonazimuth': moond['azimuth'],
        'moonilluminance_nosky': illuminance['moonilluminance_nosky'],
        'moonilluminance': illuminance['moonilluminance'],
        'moonilluminancex': illuminancex['moonilluminance'],
        'twilightilluminance0': illuminance['twilightilluminance0'],
        'twilightilluminace': illuminance['twilightilluminace'],
        'twilightilluminancex': illuminancex['twilightilluminace'],
        'totalilluminance0': illuminance['totalilluminance0'],
        'totalilluminance': illuminance['totalilluminance'],
        'totalilluminancex': illuminancex['totalilluminance']
    }
    return illuminancedata


def __some_checks__():
    """ Some simple and yet uncomplete module ckecks"""
    location = {'stat': 'LOWI', 'name': 'INNSBRUCK-AIRPORT',
                'lon': 11.35, 'lat': 47.2666667, 'elev': 593.0}
    lon = location['lon']
    __ = location['lat']
    pydattim = datetime.datetime(2015, 7, 25, 17, 0, 0)
    pydattim2 = datetime.datetime(2015, 7, 26, 5, 0, 0)
    pylocnoon, pylocmidn = py_localnoon_midnight(longitude=lon,
                                                 pytimeutc=pydattim)
    srsd = sunriseset(location, pylocnoon)
    ctd = sunriseset(location, pylocnoon, horizon=constants.TWLCIVIL)
    ntd = sunriseset(location, pylocnoon, horizon=constants.TWLNAUTIC)
    mrsd = moonriseset(location, pylocmidn)
    moonph = moonphase(pylocmidn)
    mfnd = moonfullnew(pylocmidn)
    cand = calc_astro_night(location, pydattim)
    pydattimarr = dattims.pydatetime_array(pydattim, pydattim2,
                                           d_minutes=120)
    sdict = sundata(location, pydattimarr)
    mdict = moondata(location, pydattimarr)

    print(50 * '-')
    print('Location dictionary:')
    for d in location:
        print(d, location[d])
    print(50 * '-')
    print('pydattim    = ', pydattim)
    print('pylocalnoon = ', pylocnoon)
    print('pylocalmidn = ', pylocmidn)
    print('pydattim2   = ', pydattim2)
    print(50 * '-')
    print('SUNRISE-SET Stuff')
    for d in srsd:
        print(d, srsd[d])
    print(50 * '-')
    print('CIVIL TWILIGHT Stuff')
    for d in ctd:
        print(d, ctd[d])
    print(50 * '-')
    print('NAUTICAL TWILIGHT Stuff')
    for d in ntd:
        print(d, ntd[d])
    print(50 * '-')
    print('MOONRISE-SET Stuff')
    for d in mrsd:
        print(d, mrsd[d])
    print(50 * '-')
    print('moon phase', moonph)
    print(50 * '-')
    print('MOON FULL-NEW etc..')
    for d in mfnd:
        print(d, mfnd[d])
    print(50 * '-')
    for d in cand:
        print(d, cand[d])
    print(50 * '-')
    print('pydattimarr: ', end=": ")
    for d in pydattimarr:
        print(d, end="; ")
    print()
    print(50 * '-')
    print('Sundata-Dict: ')
    for d in sdict:
        print(d, end=": ")
        [print(x, end="; ") for x in sdict[d]]
        print()
    print(50 * '-')
    print('Moondata-Dict: ')
    # print(mdict)
    for d in mdict:
        print(d, end=": ")
        [print(x, end="; ") for x in mdict[d]]
        print()
    print(50 * '-')
    # civil moon phase
    cph = civilmoonphase(mdict)
    print('Civ. Moon Phase (0..new, 1.qu, 2 full, 3.qu.): ', cph)
    print(50 * '-')
    # civil moon phase-symbol
    cphs = civilmoonphase_symbol(location, mdict)
    print('Civ. Moon Phase symbol: ', cphs)
    print(50 * '-')


if __name__ == '__main__':
    version.version('Module')
    __some_checks__()
