# Created 2016-03-01 by D.Th.
"""Provides quite a few routines for date-time manipulation and
conversion for standard datetime objects and pyephem date objects."""

from inspect import currentframe, getframeinfo
import datetime
import ephem
import sys

import adnite2.version as version
import math as m


MAXDATTIM = datetime.datetime(9999, 12, 31)  # : internal flag for ever up
MINDATTIM = datetime.datetime(1, 1, 1)  # : internal flag for ever down


def check_ddmmss(dd, mm, ss):
    c1 = (dd >= 0) and (mm >= 0) and (ss >= 0)
    c2 = (dd <= 0) and (mm <= 0) and (ss <= 0)
    return c1 or c2


def deg2dms(deg, plus='+', minus='-'):
    """Takes decimal degrees as argument. Returns the integer part of arc deg as dd,
    arc minutes as mm, and arc seconds as ss. The sign is put into the
    int sign (+/-1) and the sign character ('+'. '-'). dd, mm and ss is positve
    definite. Optional arguments plus and minus give the character
    for positive and negative sign that default to + and -"""
    sign = round(m.copysign(1, deg * 1.0))
    deg = abs(deg)
    dd = int(deg)
    m1 = (deg - dd) * 60.
    mm = int(m1)
    ss = int(round((m1 - mm) * 60.))
    if sign == -1:
        signchar = minus
    else:
        signchar = plus
    if ss == 60:
        ss = 0
        mm = mm + 1
    if mm == 60:
        mm = 0
        dd = dd + 1
    return sign, signchar, dd, mm, ss


def dms2deg(dd, mm=0, ss=0):
    """Returns decimal degrees as function of degrees dd, arc minutes mm
    and arc seconds ss. To avoid ambiguities the sign of each component has
    do be the same. Otherwise an error is returned.
    e.g.: (dd>=0, mm>=0 ss>=0) or (dd<=0 mm<=0 ss<=0)"""
    rangeok = (mm >= -60) and (mm <= 60) and (ss >= -60) and (ss <= 60)
    signok = (
        dd >= 0 and mm >= 0 and ss >= 0) or (
        dd <= 0 and mm <= 0 and ss <= 0)
    if signok and rangeok:
        return dd + mm / 60. + ss / 3600.
    else:
        if not signok:
            frameinfo = getframeinfo(currentframe())
            estr = 'ERROR in {0}.ddmmss2deg\nin linenno. {1}: dd, mm, ss are not of the same sign: {2} {3} {4}'
            return sys.exit(
                estr.format(frameinfo.filename, frameinfo.lineno, dd, mm, ss))
        if not rangeok:
            frameinfo = getframeinfo(currentframe())
            estr = 'ERROR in {0}.ddmmss2deg\nin linenno. {1}: mm, ss not within +/- 60: {2},{3}'
            return sys.exit(
                estr.format(frameinfo.filename, frameinfo.lineno, mm, ss))


def sign_of_dms(d, m, s, plus='+', minus='-'):
    """Get sign character of d, m, s"""
    if (d > 0) or (m > 0) or (s > 0):
        sign = plus
    elif (d < 0) or (m < 0) or (s < 0):
        sign = minus
    else:
        sign = ' '
    return sign


def timedelta_totalseconds(timespan):
    """Stupid work around for python 2.6 backward compatibility for
     datetime.timedelta.total_seconds()"""
    td = timespan
    result = ((td.microseconds +
               (td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10 ** 6)
    return result


def round_time(dt=None, dateDelta=datetime.timedelta(minutes=1)):
    """Round a datetime object to a multiple of a timedelta
    dt : datetime.datetime object, default now.
    dateDelta : timedelta object, we round to a multiple of this, default 1 minute.
    Author: Thierry Husson 2012 - Use it as you want but don't blame me.
            Stijn Nevens 2014 - Changed to use only datetime objects as variables
    """
    roundTo = dateDelta.total_seconds()

    if dt is None:
        dt = datetime.datetime.now()
    seconds = (dt - dt.min).seconds
    # // is a floor division, not a comment on following line:
    rounding = (seconds + roundTo / 2) // roundTo * roundTo
    return dt + datetime.timedelta(0, rounding - seconds, -dt.microsecond)


def round_datetime2minute(date_time):
    """Rounds a datetime object (exact in seconds) to the closest
    time in minutes.

    Wrapper for round_time, for compatibility reasons"""
    return round_time(date_time)


def _round_datetime2minute_(date_time):
    """Rounds a datetime object (exact in seconds) to the closest
    time in minutes.

    Not in use anymore"""
    SFORMAT = "%Y-%m-%d %H:%M"
    delta = datetime.timedelta(seconds=30)  # half a minute
    sdate_time = (date_time + delta).strftime(SFORMAT)
    return datetime.datetime.strptime(sdate_time, SFORMAT)


def str2pydatetime(datetimestr, datetimeformat="%Y-%m-%d"):
    """ Converts the string "2014-01-31" to a python datetime.date object
    with the meaning "31 Jan 2014" """
    try:
        pydatetime = datetime.datetime.strptime(datetimestr, datetimeformat)
    except Exception as e:
        print("'str2datetime'-ERROR: wrong datetime-string or -format")
        print("string: ", datetimestr)
        print("format: ", datetimeformat)
        sys.exit(e)
    return pydatetime


def pydatetime2str(pydatetime, datetimeformat="%Y-%m-%d"):
    """Converts python datetime object to string according to
    datetimeformat-string"""
    try:
        datetimestr = pydatetime.strftime(datetimeformat)
    except Exception as e:
        print("'pydatetime2str'-ERROR: wrong datetime object or strf-format")
        print("datetime: ", pydatetime)
        print("format: ", datetimeformat)
        sys.exit(e)
    return datetimestr


def pydatetime2ephemdate(pydatetime):
    """Converts a python datetime object to a pyephem date object"""
    return ephem.Date(pydatetime)


def ephemdatetime2pydatetime(ephemdatetime, roundseconds=3):
    """Converts ephem datetime to python datetime exact within
    'roundseconds' decimal digits of secondes, defaults to 3 meaning 0.001 s.
    This is for mittigating rounding errors in pypehem Date objects."""
    ed = ephem.Date(ephemdatetime)
    yy, mo, dd, hh, mi, sf0 = ed.tuple()
    sf = round(sf0, roundseconds)
    secs = datetime.timedelta(seconds=sf)  # circumvent the 60s-problem
    pydatetime = datetime.datetime(yy, mo, dd, hh, mi) + secs
    return pydatetime


def pytimedelta(dd=0, hh=0, mi=0, ss=0.0):
    """ Wrapper for datetime.timedelta"""
    return datetime.timedelta(days=dd, hours=hh, minutes=mi, seconds=ss)


def combinepydatepytime(pydate, pytime):
    """Wrapper for datetime.datetime.combine()"""
    return datetime.datetime.combine(pydate, pytime)


def pydatetime_array(datetime1, datetime2, d_days=0,
                     d_hours=0, d_minutes=0., d_seconds=0.):
    """Constructs an array/a list of python datetime objects with an
    increment of d_days,d_hours,d_minutes and d_seconds (all in integer,
    s_seconds in float). -- Constraints: datettime2 >= datetime1, all
    timeincrements >= 0
    """
    dati_arr = []
    dati = datetime1
    datidelta = abs(datetime.timedelta(days=d_days, hours=d_hours,
                                       minutes=d_minutes, seconds=d_seconds))
    while dati <= datetime2:
        dati_arr.append(dati)
        dati = dati + datidelta
    return dati_arr


def str2pydatetime_array(strarray, dattimformat="%Y-%m-%d %H:%M"):
    """Coverts a string array to a python datetime array"""
    pydattim_arr = []
    for s in strarray:
        pydt = str2pydatetime(s, datetimeformat=dattimformat)
        pydattim_arr.append(pydt)
    return pydattim_arr


def pydatestrtime2pydatetime_array(pydatetime, srttimearray,
                                   timeformat='%H:%M'):
    """Converts a python date (as datetime object) and a time string
    array to a python datetime array. Date is incresed if the time array
    jumps form 23 to 01 (or euqivalent)."""
    # datestr = pydatetime.strftime("%Y-%m-%d ")
    pydate = datetime.datetime(year=pydatetime.year,
                               month=pydatetime.month,
                               day=pydatetime.day)
    oneday = datetime.timedelta(days=1)
    pytimea = str2pydatetime_array(srttimearray,
                                   dattimformat=timeformat)
    tdmm = datetime.timedelta(hours=0)
    pydatetime_array = []
    for pt in pytimea:
        tdelta = datetime.timedelta(hours=pt.hour,
                                    minutes=pt.minute)
        if tdelta <= tdmm:
            pydate = pydate + oneday
        pydt = pydate + tdelta
        tdmm = tdelta
        pydatetime_array.append(pydt)
    return pydatetime_array


def pydatetime2ephemdatetime_array(pydattim_array):
    """Converts an array/a list of python datetime objects to
    an array / a list of pyephem dates."""
    ephemdate = [ephem.Date(d) for d in pydattim_array]
    return ephemdate


def ephemdatetime2pydatetime_arrays(ephemdatetimearray, roundit=3):
    """Converts ephem datetime arrays to python datetime arrays exact
    within oundit decimal digits of secondes, defaults to 3
    meaning 0.001 s."""
    pd_array = []
    for ed in ephemdatetimearray:
        pd = ephemdatetime2pydatetime(ed, roundit)
        pd_array.append(pd)
    return pd_array


def fine_pydatetime_array(pydatetime_coarse,
                          d_days=0,
                          d_hours=0,
                          d_minutes=0.,
                          d_seconds=0.):
    """"Make a fine python datetime array of a coarse datetime array.

    Input:

    pydatetime_coarse ... coarse datetime array
    d_hours, .., d_seconds defines the timestep

    Return:

    Fine array with equally spaced python datetime objects with its
    last element less or equal the last element of the coarse
    input array."""

    pystartdatetime = min(pydatetime_coarse)
    pyenddatetime = max(pydatetime_coarse)
    return pydatetime_array(pystartdatetime, pyenddatetime,
                            d_days, d_hours, d_minutes, d_seconds)


def py_riseset_datim2str(pydattim, csv=True):
    """Converts rise- and set- pydatetimes to a string that
    recognises everup and everdown flags"""
    if pydattim == MAXDATTIM:
        s = '++; ++:++'
    elif pydattim == MINDATTIM:
        s = '--; --:--'
    else:
        s = round_datetime2minute(pydattim).strftime('%d; %H:%M')
    return s


def pydatetime2pydatetime00(pydatetime):
    """Converts an arbitrary python datetime object to an
    python datetime object with time 00:00:00.
    eg. 2015-07-29 12:21:17 is converted to 2015-07-29 00:00:00"""
    yy = pydatetime.year
    mm = pydatetime.month
    dd = pydatetime.day
    pydate = datetime.datetime(yy, mm, dd, 0, 0, 0)
    return pydate


if __name__ == "__main__":
    version.version('Module')
    print(60 * '-')
    dtstr1 = "2015-05-01"
    dtstr2 = "2015-05-02 14:32:12"
    print("dtstr1   : ", dtstr1)
    print("dtstr2   : ", dtstr2)
    pydt1 = str2pydatetime(dtstr1, "%Y-%m-%d")
    pydt2 = str2pydatetime(dtstr2, "%Y-%m-%d %H:%M:%S")
    print("pydt1    : ", pydt1)
    print("pydt2    : ", pydt2)
    ephemdt1 = pydatetime2ephemdate(pydt1)
    ephemdt2 = pydatetime2ephemdate(pydt2)
    print("ephemdt1 : ", ephemdt1)
    print("ephemdt2 : ", ephemdt2)
    print("pydt1    : ", ephemdatetime2pydatetime(ephemdt1))
    print("pydt2    : ", ephemdatetime2pydatetime(ephemdt2))
    dstep = 12
    pydtarr = pydatetime_array(pydt1, pydt2, d_hours=dstep)
    print("pydatetime_array (step", dstep, "hours) ->")
    [print(d, end=", ") for d in pydtarr]
    print("\npydatetimearray2ephemdatetime_array ->")
    ephemdtarr = pydatetime2ephemdatetime_array(pydtarr)
    print(ephemdtarr)
    print("ephemdatetimearray2pydatetime_array ->")
    pydtarr2 = ephemdatetime2pydatetime_arrays(ephemdtarr)
    [print(d, end=", ") for d in pydtarr2]
    print()
    dfine = 5
    pyfine = fine_pydatetime_array(pydtarr, d_hours=dfine)
    print('Pyfine (step ', dfine, ' hours) ->')
    [print(d, end=", ") for d in pyfine]
    print()
    print(60 * "-")
    print('Test py_riseset_datim2str')
    print('Ever-Up   : ', MAXDATTIM)
    print('Ever-Up  -> ', py_riseset_datim2str(MAXDATTIM))
    print('Ever-Up   : ', MINDATTIM)
    print('Ever-Up  -> ', py_riseset_datim2str(MINDATTIM))
    print()
    print(60 * "-")
    timestra = ['19:00', '22:00', '00:00', '01:00', '04:00',
                '19:00', '23:00', '00:00', '05:00']
    print('Timestring array: ', timestra)
    pytimea = str2pydatetime_array(timestra,
                                   dattimformat="%H:%M")
    print('str2pydatetime_array: ')
    [print(pt.strftime('%H:%M'), end=" ") for pt in pytimea]
    print()
    pydatetimearray2 = pydatestrtime2pydatetime_array(
        pydt1,
        timestra)
    print('pydatestrtime2pydate_array: ')
    [print(dt) for dt in pydatetimearray2]
    print(60 * '-')
    print('round_datetime2minute: ')
    ttx = datetime.datetime(2015, 11, 0o3, 15, 32, 29)
    print('Exact to second      : ', ttx)
    ttr = round_datetime2minute(ttx)
    print('Rounded to one minute: ', ttr)
    ttx = datetime.datetime(2015, 11, 0o3, 15, 32, 30)
    print('Exact to second      : ', ttx)
    ttr = round_datetime2minute(ttx)
    print('Rounded to one minute: ', ttr)
    ttx = datetime.datetime(2015, 11, 0o3, 15, 32, 31)
    print('Exact to second      : ', ttx)
    ttr = round_datetime2minute(ttx)
    print('Rounded to one minute: ', ttr)
    print(60 * '-')
