# Created 2016-03-01 by D.Th.
"""Graphic output"""

import datetime
import sys

import adnite2.astrophysics as astrophysics
import adnite2.checker as checker
import adnite2.constants as constants
import adnite2.dattims as dattims
import adnite2.db as db
import adnite2.inout as inout
import adnite2.version as version
import matplotlib.pyplot as plt
import numpy as np
import os.path as op

FOOTERFONTSIZE = 7  # : size of footer line in graphics
GRAPH_DPI = 150  # : dpi of output graphics
A5_LANDSCAPE = (21. / 2.54, 14.8 / 2.54)
GRAPH_FORMAT = A5_LANDSCAPE


def plot_illuminances(stationdata, parameters,
                      illuminancedata, relevantdata,
                      xkcd=False):
    """Plot graphic with illuminances

    * stationdata .. dictionary with station data
    * parameters ... dictionary with input parameters
    * illuminacadata ... dictionary with illuminance-arrays to be plot
    * xkcd ... if True, plots are done in the sloppy xkcd-style of matplotlib

    Returns ... full path name of output graphic
    """
    # P r e p a r e   p l o t t i n g
    stat = stationdata
    # twilights in ephem-date-format
    ephemenet = dattims.pydatetime2ephemdate(relevantdata['enet'])
    ephembnmt = dattims.pydatetime2ephemdate(relevantdata['bnmt'])
    startpydate = illuminancedata['pydatetimes'][0]
    endpydate = illuminancedata['pydatetimes'][-1]
    title = 'Illuminances ' + stat['station'] + ' (' + stat['name'] + ')\n'
    title = title + startpydate.strftime('%Y-%m-%d %H:%M') + ' - '
    title = title + endpydate.strftime('%Y-%m-%d %H:%M UTC')
    ephemdattim = illuminancedata['ephemdatetime']
    startephemdattim = ephemdattim[0]
    startephemtime, __ = np.modf(startephemdattim)
    pldates = ((ephemdattim - startephemdattim) * 24.0) + \
        startephemtime * 24 + 12.
    # twilight in x-coordinates for graphic
    xenet = ((ephemenet - startephemdattim) * 24.0) + \
        startephemtime * 24 + 12.
    xbnmt = ((ephembnmt - startephemdattim) * 24.0) + \
        startephemtime * 24 + 12.
    # xticks
    xtickdata = np.arange(pldates[0], pldates[-1] + 1)
    xticklabel = []
    for x in xtickdata:
        x1 = x % 24
        xticklabel.append("{0:02.0f}".format(x1))
    illumlabels = [0.1, 1, 10, 100, 1000, 10000]
    # P l o t t i n g
    fig = plt.figure(figsize=GRAPH_FORMAT, facecolor='lightyellow')  # A5
    if xkcd:
        # XKCD - Comic style
        plt.xkcd()
    plt.suptitle(title, fontsize=14)
    plt.figtext(0.05, 0.01, version.__copyright__,
                size=FOOTERFONTSIZE,
                horizontalalignment='left',
                fontweight='normal')
    plt.semilogy(pldates, illuminancedata['totalilluminance'] * 1e3,
                 lw=3, c='b', label='Main WX')
    plt.semilogy(pldates, illuminancedata['totalilluminancex'] * 1e3,
                 lw=3, c='r', label='Alternate WX')
    plt.axvline(xenet, c='b', ls='--', label='Nautical twilight')
    plt.axvline(xbnmt, c='b', ls='--')
    # for legend positioning reasons better plot than plt.axhline() :-)
    plt.semilogy(
        pldates,
        np.ones(len(pldates)) * constants.MILILUXALERT_LOWER * 1000,
        c='r', ls='--', lw=3,
        label="{0:3.1f} mlx".format(constants.MILILUXALERT_LOWER * 1000)
    )
    plt.ylim(1e-1, 1e4)
    plt.xlim(pldates[0], pldates[-1])
    plt.xticks(xtickdata, xticklabel)
    plt.yticks(illumlabels, illumlabels)
    plt.xlim(pldates[0], pldates[-1])
    plt.xlabel('Time [UTC]')
    plt.ylabel('Illuminance [mlx]')
    plt.grid()
    lg = plt.legend(
        loc='upper center',
        fancybox=True,
        framealpha=0.5)
    fr = lg.get_frame()
    fr.set_facecolor('lightyellow')
    plt.minorticks_on()
    # D e t e r m i n e   o u t p u t f i l e
    fname, outdir, __ = inout.define_outfilename(parameters,
                                                 basename='illum_goutfilen',
                                                 extension='png')
    # print("--> inout.plot_illuminances: ", fname)   # ########
    goutfname = op.join(outdir, fname)
    try:
        f = open(goutfname, 'w')
    except IOError as e:
        print(
            "ERROR in 'inout.plot_illuminances': ",
            "file cannot be opend for writing",
            file=sys.stderr)
        print("Does directory '", outdir, "' exist?", file=sys.stderr)
        sys.exit(e)
    f.close()
    # s a v e   g r a p h i c   i n   f i l e
    fig.savefig(goutfname, facecolor='lightyellow', dpi=GRAPH_DPI)
    return goutfname


def plot_moon(stationdata, parameters, basicdata,
              moondata_coarse, moondata_fine, moondata_verycoarse,
              moonfillstyle='none', xkcd=False):
    """Plot graphic with moon positions and related stuff

    * stationdata .. dictionary with station data
    * parameters ... dictionary with input parameters
    * moondata_coarse ... dictionary with moon data arrays to be plot (coarse)
    * moondata_fine ... dictionary with moon data arrays to be plot (fine)
    * moonfillstyle ... fillstyle for the symbolic moon icon
                        (as returned by astrophysics.civilmoonphase_symbol)
    * xkcd ... if True, plots are done in the sloppy xkcd-style of matplotlib

    Returns ... full path name of output graphic
    """
    print('moonfillstyle in plot_moon: ', moonfillstyle)
    MOONALTCOLOR = 'b'
    MOONRELILLCOLOR = 'coral'
    MOONAZIMUTHCOLOR = 'b'

    # Determine outputfile
    fname, outdir, __ = inout.define_outfilename(parameters,
                                                 basename='moon_goutfilen',
                                                 extension='png')
    goutfname = op.join(outdir, fname)
    try:
        f = open(goutfname, 'w')
    except IOError as e:
        print(
            "ERROR in 'graphics.plot_moon': ",
            "file cannot be opend for writing",
            file=sys.stderr)
        print("Does directory '", outdir, "' exist?", file=sys.stderr)
        sys.exit(e)
    f.close()
    # Prepare Plotting
    # Fine data
    moondata = moondata_fine  # for convenience
    stat = stationdata
    startpydate = moondata['pydatetimes'][0]
    endpydate = moondata['pydatetimes'][-1]
    ephemdattim = moondata['ephemdatetime']
    startephemdattim = ephemdattim[0]
    startephemtime, __ = np.modf(startephemdattim)
    malt_deg = np.rad2deg(moondata['altitude'])
    pldates = ((ephemdattim - startephemdattim) * 24.0) + \
        startephemtime * 24 + 12.
    # Coarse data
    ephemdattim_coarse = moondata_coarse['ephemdatetime']
    startephemdattim_coarse = ephemdattim_coarse[0]
    startephemtime_coarse, __ = np.modf(
        startephemdattim_coarse)
    pldates_coarse = ((ephemdattim_coarse - startephemdattim_coarse) * 24.0)\
        + startephemtime_coarse * 24 + 12.
    moonrelill_coarse = moondata_coarse['rel_illuminance_ESAA2013'] * 100.0
    # Very coarse data
    ephemdattim_verycoarse = moondata_verycoarse['ephemdatetime']
    startephemdattim_verycoarse = ephemdattim_verycoarse[0]
    startephemtime_verycoarse, __ = np.modf(
        startephemdattim_verycoarse)
    pldates_verycoarse = ((
        (ephemdattim_verycoarse -
         startephemdattim_verycoarse) * 24.0) +
        startephemtime_verycoarse * 24 + 12.)
    azimuths_verycoarse = np.rad2deg(moondata_verycoarse['azimuth'])
    altitudes_verycoarse = np.rad2deg(moondata_verycoarse['altitude'])
    title = 'Moon position ' + stat['station'] + ' (' + stat['name'] + ')\n'
    title = title + startpydate.strftime('%Y-%m-%d %H:%M') + ' - '
    title = title + endpydate.strftime('%Y-%m-%d %H:%M UTC')
    xtickdata = np.arange(pldates[0], pldates[-1] + 1)
    xticklabel = []
    for x in xtickdata:
        x1 = x % 24
        xticklabel.append("{0:02.0f}".format(x1))
    y_alt_tickdata = np.arange(-15, 90 + 15, 15)
    y_relill_tickdata = y_alt_tickdata
    # Plotting
    fig = plt.figure(
        figsize=GRAPH_FORMAT,
        facecolor='LightYellow')  # A5
    if xkcd:
        # XKCD - Comic style
        plt.xkcd()
    plt.suptitle(title, fontsize=14)
    plt.figtext(0.05, 0.01, version.__copyright__,
                size=FOOTERFONTSIZE,
                horizontalalignment='left',
                fontweight='normal')
    # horizon (alt=0)
    plt.plot(
        pldates,
        np.ones(len(pldates)) * 0,
        c=MOONALTCOLOR, ls=':', lw=2,
        label="Horizon")
    # moon altitudes
    plt.plot(pldates, malt_deg, lw=3, c=MOONALTCOLOR)
    plt.xlim(pldates[0], pldates[-1])
    plt.xticks(xtickdata, xticklabel)
    plt.xlim(pldates[0], pldates[-1])
    plt.xlabel('Time [UTC]')
    plt.ylim(-15, 90)
    plt.yticks(y_alt_tickdata, y_alt_tickdata)
    plt.ylabel('Moon altitude [deg]', color=MOONALTCOLOR)
    plt.grid()
    plt.minorticks_on()
    ax1 = plt.gca()
    for tl in ax1.get_yticklabels():
        tl.set_color(MOONALTCOLOR)
    # moon rel. illum. (coarse)
    ax2 = ax1.twinx()
    ax2.plot(pldates_coarse, moonrelill_coarse, ls='', marker='o',
             markersize=10, alpha=0.75, fillstyle=moonfillstyle,
             color=MOONRELILLCOLOR)
    ax2.set_ylabel('rel. moon illuminance [%]', color=MOONRELILLCOLOR)
    ax2.set_ylim(-15, 90)
    ax2.set_xlim(pldates[0], pldates[-1])
    plt.yticks(y_relill_tickdata, y_relill_tickdata)
    for tl in ax2.get_yticklabels():
        tl.set_color(MOONRELILLCOLOR)
    plt.minorticks_on()
    # plot directions 5 degrees above altitudes every hour
    kk = 0
    for xt in pldates_verycoarse:
        alt = altitudes_verycoarse[kk]
        az = azimuths_verycoarse[kk]
        if alt > 0:
            ax1.text(xt, alt + 5, "{0:03.0f}".format(az), ha='center',
                     color=MOONAZIMUTHCOLOR, size='medium')
        kk = kk + 1
    # plot some base infos
    nnm = basicdata['nextnewmoon']
    nfm = basicdata['nextfullmoon']
    phasetxt = "Moon phase {0:2.0f} %".format(basicdata['moonphase'])
    nextnew = "Next new moon: " + \
        dattims.ephemdatetime2pydatetime(nnm).strftime("%Y-%m-%d")
    nextfull = "Next full moon: " + \
        dattims.ephemdatetime2pydatetime(nfm).strftime("%Y-%m-%d")
    textstr = phasetxt + '\n' + nextnew + '\n' + nextfull
    props = dict(boxstyle='round', facecolor='lightyellow', alpha=0.5)
    plt.figtext(0.56, 0.86, textstr, transform=ax1.transAxes, fontsize=13,
                verticalalignment='top', bbox=props)
    # save graphic in file
    fig.savefig(goutfname, facecolor='lightyellow', dpi=GRAPH_DPI)
    return goutfname


def __main__(parameters):
    """Main entry point for the script."""
    XKCD = constants.XKCD_PLOT  # define sloppy (True) or exact (False) print
    print(constants.SEPARATOR1)
    print("Running 'adnite2.graphics.py'")

    try:
        stationdb = parameters['stationdb']
    except KeyError as e:
        print("ERROR in graphics.py", file=sys.stderr)
        print("Cannot find valid database entry in inputfile",
              file=sys.stderr)
        sys.exit(e)
    try:
        station = parameters['station'].upper()
    except KeyError:
        print("WARNING: no 'station' in input file",
              file=sys.stderr)
        print("Station set to " + constants.DEFAULTSTATION,
              file=sys.stderr)
        station = constants.DEFAULTSTATION
        parameters['station'] = station
    stationdata = db.read_single_station(station, stationdb)
    try:
        pydate1 = dattims.str2pydatetime(parameters['date1'])
    except KeyError:
        print("No start date found, today's date is taken")
        now = datetime.datetime.utcnow()
        pydate1 = dattims.pydatetime2pydatetime00(now)

    try:
        pydatetimes = dattims.pydatestrtime2pydatetime_array(
            pydate1,
            parameters['times'],
            timeformat='%H:%M')
    except Exception as e:
        print('ERROR in graphics.py: Cannot not correctly convert',
              file=sys.stderr)
        print(parameters['times'],
              file=sys.stderr)
        print('Wrong format?', files=sys.stderr)
        sys.exit(e)

    print('Station      : ', station)
    print('Date         : ', pydate1.strftime("%Y-%m-%d"))
    for d in pydatetimes:
        print(' descr. times: ', d.strftime('%b-%d  %H:%M:%S'))
    timespan = pydatetimes[-1] - pydatetimes[0]
    # timesp_min = datetime.timedelta.total_seconds(timespan) // 60
    timesp_min = dattims.timedelta_totalseconds(timespan) // 60
    print('Timespan     : {0:} ({1:.0f} minutes)'.format(timespan, timesp_min))
    print('Timestep     : {0:} minutes'.format(constants.TIMESTEP4GRAPHS))

    if checker.checkcloudandgrounddata(parameters):
        print('Cloud and ground input data OK')
    print(constants.SEPARATOR2)
    # illuminances
    print('Calculate illuminances and related stuff ...')
    ill_data = astrophysics.illuminancedata_fine(
        stationdata,
        pydatetimes,
        params=parameters,
        timestep=constants.TIMESTEP4GRAPHS)
    print('Plot illuminance graphic')
    # astro basics
    basic_data = astrophysics.calc_astro_night(stationdata, pydate1)
    relev_data = astrophysics.extract_relevant_astro_night(basic_data)
    goutfname = plot_illuminances(stationdata, parameters,
                                  ill_data, relev_data, xkcd=XKCD)
    print("Illumination graphic output saved to:\n" + goutfname)
    print(constants.SEPARATOR2)
    print('Calculate moon positions and related stuff ...')
    # coarse moon data (30 min)
    pydatetimes_coarse = dattims.fine_pydatetime_array(
        pydatetimes, d_minutes=constants.TIMESTEP4COARSEGRAPHS)
    moondata_coarse = astrophysics.moondata(stationdata, pydatetimes_coarse)
    # fine moon data (2 min)
    pydatetimes_fine = dattims.fine_pydatetime_array(
        pydatetimes, d_minutes=constants.TIMESTEP4GRAPHS)
    moondata_fine = astrophysics.moondata(stationdata, pydatetimes_fine)
    # very coarse moon data (60 min)
    pydatetime_verycoarse = dattims.fine_pydatetime_array(
        pydatetimes, d_minutes=constants.TIMESTEP4VERYCOARSEGRAPHS)
    moondata_verycoarse = astrophysics.moondata(
        stationdata, pydatetime_verycoarse)
    # Plot fine and coarse data
    print('Plot moon graphic')
    #           Moon style for marker
    moonfstyle = astrophysics.civilmoonphase_symbol(stationdata,
                                                    moondata_coarse)
    goutfname = plot_moon(stationdata, parameters, basic_data,
                          moondata_coarse, moondata_fine,
                          moondata_verycoarse,
                          moonfillstyle=moonfstyle,
                          xkcd=XKCD)
    print("Moon graphic output saved to:\n" + goutfname)
    print(constants.SEPARATOR0)
    return


if __name__ == '__main__':
    version.version('Module')
