# Created 2016-03-01 by D.Th.
"""Contains constants and related stuff"""

import csv
import sys

from adnite2 import USER_CONFIG_DIRECTORY
import adnite2.version as version
import math as m
import os.path as op


# Separatorlines
SEPARATOR0 = 79 * '*'
SEPARATOR1 = 79 * '='
SEPARATOR2 = 79 * '-'

USER_CONFIGFILENAME = op.join(USER_CONFIG_DIRECTORY, 'default.cfg')
USER_TABLE_FORM = op.join(USER_CONFIG_DIRECTORY, 'table.form')

TIMESTEP4GRAPHS = 2  # : timestep for graphics in minutes
TIMESTEP4COARSEGRAPHS = 30  # : timestep for coarse graphics in minutes
# : timestep for very coarse graphics in minutes
TIMESTEP4VERYCOARSEGRAPHS = 60

# Lists of allowed tokens in parameter-inputfile
# for the function 'read_inputparms'  etc...
PARAMSTRINGS = ('stationdb', 'station',
                'outdir',
                'basics_outfilen', 'moon_outfilen', 'illum_outfilen',
                'moon_goutfilen', 'illum_goutfilen',
                'rsttable', 'rsttemplate', 'rstfile',
                'date1', 'date2')
PARAMLISTS = ('times', 'cloudsn', 'cloudsnx',
              'cloudsd', 'cloudsdx',
              'ground', 'groundx')
PARAMS_ALL_ALLOWED = PARAMSTRINGS + PARAMLISTS

CONFIG_PARAM_STRINGS = ('workdir', 'stationdb', 'htmldir', 'inputfile',
                        'viewer', 'pdfviewer', 'browser', 'adnite',
                        'rst2html', 'rst2pdf', 'helpsite',
                        'illum_deepnight', 'illum_sunset', 'illum_civiltwilight',
                        'illum_alert_lower', 'illum_alert_higher',
                        'progname', 'titlegui', 'logopath',
                        'xkcd',
                        'fontsize0', 'fontsize1', 'fontsize2')
CONFIG_PARAM_LISTS = ('favorites',
                      'cloudamount', 'cloudtypes', 'cloudalbedos', 'cloudnames',
                      'groundtypes', 'groundalbedos', 'groundnames')

# M e t - A s t r o - P h y s i c a l  variables and constants
DEG2RAD = m.pi / 180.0
# SUN and TWILIGHTS (fits USNO-Data)
SUNRISESET = '-0:50'  # :sun radius 0°16' + standard atmosph. refraction 0°34'
TWLCIVIL = ' -6:00'  # :sun height civil twilight -6°
TWLNAUTIC = '-12:00'  # :sun height nautical twilight -12°
TWLASTRO = '-18:00'  # :sun height astronomical twilight -18°
HCIVIL = -6.0  # sun altitude for civil
HNAUTIC = -12.0  # sun altitude for nautical
HASTRONOMIC = -18.0  # sun altitude for austronomical
# http://en.wikipedia.org/wiki/Apparent_magnitude
MAG_FULLMOON_MEAN = -12.74  # : mean magnitude of full moon
ILLUM_FULLMOON_MEAN = 0.265  # : calculateted illum. mean full moon in zenith

# Atmospher. -  T u r b i d i t y
Q_TRANSPARENCYDICT = {
    'very clear': 0.95,
    'clear': 0.90,
    'medium': 0.85,
    'turbid': 0.80,
    'very turbid': 0.75}
Q_TRANSPARENCY = Q_TRANSPARENCYDICT[
    'medium']  # : transparency  (1: full transparency, 0: full opacity)
# Atmosph. albedo
ALBEDO_ATMOS = {
    'RAL': 0.07,  # Rayleigh
    'NRM': 0.15,  # normal turbidity
    'TRB': 0.25  # high turbidity
}  # : Albedo of the clear sky, simplified from Möller, 1965

# ####################################################################


def read_parameters(parmfile,
                    allowedstrings=PARAMSTRINGS,
                    allowedlists=PARAMLISTS):
    """Read input parameters from an user interface file for adnite2"""
    # print('parmfile: ', parmfile)

    def open_p2_or_p3(filename):
        if sys.version_info[0] < 3:
            # python2
            infile = open(filename, 'rb')
        else:
            # python3
            infile = open(filename, 'r', newline='', encoding='utf8')
        return infile

    def parm_error():
        print("ERROR: Problems with parameter '" + str(kk) + "'",
              file=sys.stderr)
        print("Please check parameter file ", parmfile,
              file=sys.stderr)
        print("Most likely a key word without ';' ",
              "followed by an argument", file=sys.stderr)
        print("System error message --> ", file=sys.stderr)
        sys.exit(e)
        return None
    try:
        with open_p2_or_p3(parmfile) as f:
            # read input
            inlines = csv.reader(f, delimiter=str(';'))
            parms = {}
            for row in inlines:
                if len(row) > 0:  # only non empty lines
                    dl = row[0].strip()
                    if not dl.startswith('#'):  # only non comment lines
                        parms[dl] = row[1:]
        f.close()
    except IOError as e:
        print("ERROR: problems in '" + __name__ + "' with parameter file ",
              parmfile, file=sys.stderr)
        print(e, file=sys.stderr)
        sys.exit(e)
    # raw process input params
    all_allowed = allowedstrings + allowedlists
    for kk in parms:
        if kk in allowedstrings:  # strings
            # strings
            try:
                parms[kk] = parms[kk][0].strip()
            except IndexError as e:
                parm_error()
        if kk in allowedlists:  # list of strings
            try:
                for ll in range(len(parms[kk])):
                    parms[kk][ll] = parms[kk][ll].strip()
            except IndexError as e:
                parm_error()
    # clean from dictionary entries that are NOT in
    # all_allowed strings
    parmcp = {}
    for k in parms:
        if k in all_allowed:
            parmcp[k] = parms[k]
        else:
            txt = "WARNING: in file '{0:}' irregular param. '{1:}:{2:}' IGNORED"
            print(txt.format(parmfile, k, parms[k], file=sys.stderr))
    return parmcp
# ####################################################################


configparams = read_parameters(
    USER_CONFIGFILENAME,
    CONFIG_PARAM_STRINGS,
    CONFIG_PARAM_LISTS)

#  P a r a m e t e r s  r e a d   f r o m   c o n f i g - f i l e
try:
    XKCD_PLOT = (int(configparams['xkcd']) == 1)
except (KeyError, ValueError):
    XKCD_PLOT = False
try:
    DEFAULTSTATION = configparams['favorites'][0]
except (KeyError, ValueError):
    DEFAULTSTATION = 'LOWL'  # : default station
try:
    LOGOPATH = op.expanduser(configparams['logopath'])
except (KeyError, ValueError):
    LOGOPATH = op.expanduser('~/.adnite2/logo/logo.png')
try:
    PROGRAMNAME = configparams['progname']
except (KeyError, ValueError):
    PROGRAMNAME = 'AdNite2'
try:
    TITLEGUI = configparams['titlegui']
except (KeyError, ValueError):
    TITLEGUI = 'AdNite2 - Night Vision Tool'
try:
    BACKGROUND_ILLUMINATION = float(configparams['illum_deepnight'])
except (KeyError, ValueError):
    # SKC estimate (Urban/Seidelmann 2013)
    BACKGROUND_ILLUMINATION = 2.0e-3
try:
    SUNSET_ILLUMINATION = float(configparams['illum_sunset'])
except (KeyError, ValueError):
    SUNSET_ILLUMINATION = 7.5e2  # : lx , rough estimate sunset clear sky
try:
    TWLCIVIL_ILLUMINATION = float(configparams['illum_civiltwilight'])
except (KeyError, ValueError):
    TWLCIVIL_ILLUMINATION = 3.  # : lx, rough estimate clear sky
try:
    MILILUXALERT_LOWER = float(configparams['illum_alert_lower'])
except (KeyError, ValueError):
    MILILUXALERT_LOWER = 0.5e-3  # : lower illuminance alert in lux
try:
    MILILUXALERT_UPPER = float(configparams['illum_alert_upper'])
except (KeyError, ValueError):
    MILILUXALERT_UPPER = 1.5e-3  # : upper illuminance alert in lux
try:
    CLOUDAMOUNTS = [float(n) for n in configparams['cloudamount']]
except (KeyError, ValueError):
    CLOUDAMOUNTS = range(0, 11)
CLOUDAMOUNT_MAX = max(CLOUDAMOUNTS)
CLOUDNCLOUDS = CLOUDAMOUNTS  # synonymous
try:
    CLOUDDENSITIES = list(configparams['cloudtypes'])
except (KeyError, ValueError):
    CLOUDDENSITIES = list(('0', '1', '2', '3'))
try:
    CLOUDNAMES = list(configparams['cloudnames'])
    # print('constants.CLOUDANEMS', CLOUDNAMES)
except (KeyError, ValueError):
    CLOUDNAMES = list(('frsh sn', 'old sn', 'frozen', 'dry', 'humid', 'wet'))
try:
    CLOUDALBEDOS = [float(ca) for ca in configparams['cloudalbedos']]
except (KeyError, ValueError):
    CLOUDALBEDOS = [0.3, 0.5, 0.7, 0.9]
ALBEDO_CLOUD = dict(
    zip(configparams['cloudtypes'], CLOUDALBEDOS))
NAME_CLOUD = dict(zip(CLOUDDENSITIES, CLOUDNAMES))

try:
    GROUNDDATA = configparams['groundtypes']
except (KeyError, ValueError):
    GROUNDDATA = list(
        ('DRY',
         'HUM',
         'FRZ',
         'WET',
         'SNF',
         'SNO',
         'SN0'))  # 'sn0' = 'sno'
try:
    GROUNDALBEDO = [float(ga) for ga in configparams['groundalbedos']]
except (KeyError, ValueError):
    # GROUNDALBEDOS, simplified from Nielsson, 2007
    GROUNDALBEDO = [0.80, 0.50, 0.35, 0.25, 0.10, 0.05]
try:
    GROUNDNAMES = list(configparams['groundnames'])
    # print('constants.GROUNDNAMES', GROUNDNAMES)
except (KeyError, ValueError):
    GROUNDNAMES = list(('frsh sn', 'old sn', 'frozen', 'dry', 'humid', 'wet'))
ALBEDO_GROUND = dict(
    zip(configparams['groundtypes'], GROUNDALBEDO))
NAME_GROUND = dict(zip(GROUNDDATA, GROUNDNAMES))
# print('constants.NAME_GROUND', NAME_GROUND)

# C a l c u l a t e d   t o o l   t i p s
TOOLTIP_CLOUDN = """Enter clouds in 1/{0:2.0f} of the total coverage (0..{0:2.0f}).""".format(
    CLOUDAMOUNT_MAX)
TOOLTIP_CLOUDDENS = """Enter cloud density (corresponding albedo values in brackets):\n"""

for k in range(len(CLOUDDENSITIES)):
    c = "\n{:1s} ... {:5s} ({:})\n".format(
        CLOUDDENSITIES[k],
        configparams['cloudnames'][k],
        CLOUDALBEDOS[k])
    TOOLTIP_CLOUDDENS = TOOLTIP_CLOUDDENS + c

TOOLTIP_GROUNDDATA = """Enter ground type (corresponding albedo values in brackets):\n"""
for k in range(len(GROUNDDATA)):
    c = "\n{:1s} ... {:5s} ({:})\n".format(
        GROUNDDATA[k],
        configparams['groundnames'][k],
        GROUNDALBEDO[k])
    TOOLTIP_GROUNDDATA = TOOLTIP_GROUNDDATA + c

if __name__ == "__main__":
    version.version('Module')
