# Created 2016-03-01 by D.Th.
"""General input-output routines"""

import datetime
import os
import sys

import adnite2.constants as constants
import adnite2.dattims as dattims
import adnite2.version as version
import numpy as np
import os.path as op


def create_dirs(deepdirs):
    """Creates recursively deep directories and does nothing if they exits already.

    deepdir -- deep directory names, e.g. /home/theuser/this/is/a/deep/directory

    returns deepdir if successful, otherwise the exception content (OSError). Most likely:
    a 'Permission denied' - error.
    """
    # Very simple in python3.4
    try:
        os.makedirs(deepdirs, exist_ok=True)
        return deepdirs
    except OSError as e:
        return e


def trunc_str(s, length=80, ellipse='..'):
    """Truncate the string s to lenght length including the ellipse. If the string
    is shorter den length then do nothing."""
    le = len(ellipse)
    ls = length - le
    return s[:ls] + (s[ls:] and ellipse)


def format_illum_floats(x):
    """Format mililux illuminances"""
    if x >= 1000.:
        s = '9999'
    elif x >= 20.:
        s = "{0:4.0f}".format(x)
    else:
        s = "{0:4.1f}".format(x)
    return s


def fine_reals(pydatetime_coarse, reals_coarse, pydatetime_fine):
    """Interpolates from a coarse real data array to a fine one."""
    # Ensure input strings are converted to float
    for k, v in enumerate(reals_coarse):
        reals_coarse[k] = float(v)
    ephemdate = dattims.pydatetime2ephemdatetime_array(pydatetime_coarse)
    ephemdate_fine = dattims.pydatetime2ephemdatetime_array(pydatetime_fine)
    # print('In inout.fine_reals: ehpehmdate_fine --> ', ephemdate_fine)          #######################
    # print('In inout.fine_reals: ephemdate --> ', ephemdate)          #######################
    # print('In inout.fine_reals: reals_coarse --> ', reals_coarse)          #######################
    reals_fine = np.interp(ephemdate_fine, ephemdate, reals_coarse)
    return reals_fine


def print_parameters(parms, header=None, outfile=sys.stdout):
    """Alphapetically sorted list of all parameters:"""
    if not (header is None):
        hl = header + ': ' + print_parameters.__doc__
    else:
        hl = print_parameters.__doc__
    print(hl, file=outfile)
    for d in sorted(parms.keys()):
        print("    {0:12s}: ".format(d), parms[d], file=outfile)
    outfile.flush()
    return


def save_parameters(parameters,
                    allowedstrings=constants.PARAMSTRINGS,
                    allowedlists=constants.PARAMLISTS, outfile=sys.stdout):
    """Saves semicolon separated parameter dictionary to a file object.

    The key is sorted alphanumeric.
    """
    for d in sorted(parameters):
        if d in allowedstrings:
            print("{0:s}; {1:s}".format(d, parameters[d]), file=outfile)
        elif d in allowedlists:
            print("{0:s}".format(d), end="", file=outfile)
            for p in parameters[d]:
                print("; {0:s}".format(p), end='', file=outfile)
            print(file=outfile)
    return None


def define_outfilename(parameters, basename='basics', extension='csv'):
    """Determine the outfilename and outputfirectory.
    If defined the values of parameters-dictionary are used.
    Otherwise the names are calculated with the use of the
    actual request date, the station name, and the variable
    'basename' combined with a file extension

    * parameters ... dictionary with imput parameters
    * basename ... outputfile keyword of parameters input file,
      defaults to 'basicsoutfn'
    * extension ... filename extension, defaults to 'csv'

    Returns tuple with:

    * outfile  ... path name of the outfile
    * outdir ... name of the outrputdirectory
    * pydate1 ... python datetime object with the actual date taken
    """
    paramstr = basename
    try:
        pydate1 = datetime.datetime.strptime(
            parameters['date1'], "%Y-%m-%d")
    except KeyError:
        pydate1 = datetime.datetime.utcnow()

    try:
        fname = parameters[paramstr]
    except KeyError:
        fname = basename + '_' + parameters['station'] + \
            pydate1.strftime('_%Y%m%d.' + extension)
    try:
        outdir = parameters['outdir']
    except KeyError:
        outdir = './'
    outfile = op.join(op.expanduser(op.abspath(outdir)), fname)
    return outfile, outdir, pydate1


def output_rst_table(pydatetimearray,
                     parameters,
                     stationdata,
                     relevantdata,
                     moondata,
                     illumdata):
    """Outputs an restructuredText-Table with basic, moon and illumination
    data to a file."""
    #  D e f i n e   f o r m a t  strings, header lines, etc ...
    timefmt = "%d %H:%M    "
    timefmt2 = "%d %H:%M "
    datefmt2 = " %Y-%m-%d     "
    dummytime = "-- --:--    "
    tablesunl1 = 7 * "=========== "
    tablesunheaders = [' Date', '  Sun set  ', 'ECET', 'ENET',
                       'BNMT', 'BCMT', 'Sun rise ']
    th = ["{0:12s}".format(d) for d in tablesunheaders]
    tablesunh1 = "".join(th)
    tablemoonl1 = 6 * (15 * "=" + " ")
    tablemoonheaders = ['Date ', 'Moon       ', 'Moon          ', ' Phase',
                        'Full moon', 'New moon']
    th = ["{0:16s}".format(d) for d in tablemoonheaders]
    tablemoonh1 = "".join(th)
    #  D i c t i o n a r i e s
    aufunter = {'r': '  r', 's': '   s'}
    wolkendichte = constants.NAME_CLOUD
    # wolkendichte = {'0': '  thin', '1': 'thn.m.', '2': 'thk.m.', '3': ' thick'}
    bodenzustand = constants.NAME_GROUND
#     bodenzustand = {'dry': '    dry',
#                     'hum': '  humid',
#                     'frz': ' frozen',
#                     'wet': '    wet',
#                     'snf': 'frsh sn',
#                     'sno': ' old sn'}
    rvd = relevantdata
    # [print(r, rvd[r]) for r in rvd]
    rsttablen = op.join(parameters['outdir'], parameters['rsttable'])
    #  O p e n   f i l e
    try:
        f = open(rsttablen, mode='w')
    except IOError as e:
        print("ERROR in 'inout.output_rst_table'", file=sys.stderr)
        print("Cannot open file for writing", file=sys.stderr)
        print(rsttablen, file=sys.stderr)
        print("No write permission? Directory not existing?", file=sys.stderr)
        sys.exit(e)
    # H e a d l i n e
    header0 = "{0:} ({1:}) - {2:}".format(stationdata['station'],
                                          stationdata['name'],
                                          parameters['date1'])
    print(header0, file=f)
    print(79 * '=' + '\n', file=f)

    #  S u n   a n d   T w i l i g h t
    uff0 = "Sun, moon and illuminance tables"
    print(uff0, file=f)
    print(79 * "-" + "\n", file=f)
    print("1. Sun  \n\n", file=f)
    print(tablesunl1, file=f)
    print(tablesunh1, file=f)
    print(tablesunl1, file=f)
    print(parameters['date1'], file=f, end="  ")
    try:
        print(dattims.round_datetime2minute(rvd['sunset']).strftime(timefmt),
              file=f, end="")
    except:
        print(dummytime, file=f, end="")
    try:
        print(dattims.round_datetime2minute(rvd['ecet']).strftime(timefmt),
              file=f, end="")
    except:
        print(dummytime, file=f, end="")
    try:
        print(dattims.round_datetime2minute(rvd['enet']).strftime(timefmt),
              file=f, end="")
    except:
        print(dummytime, file=f, end="")
    try:
        print(dattims.round_datetime2minute(rvd['bnmt']).strftime(timefmt),
              file=f, end="")
    except:
        print(dummytime, file=f, end="")
    try:
        print(dattims.round_datetime2minute(rvd['bcmt']).strftime(timefmt),
              file=f, end="")
    except:
        print(dummytime, file=f, end="")
    try:
        print(dattims.round_datetime2minute(rvd['sunrise']).strftime(timefmt),
              file=f, end="")
    except:
        print(dummytime, file=f, end=" ")
    print(file=f)
    print(tablesunl1, file=f)
    print("\n\n", file=f)

    #  M o o n
    print("\n2. Moon\n", file=f)
    print(tablemoonl1, file=f)
    print(tablemoonh1, file=f)
    print(tablemoonl1, file=f)

    print("{0:15s}".format(parameters['date1']), file=f, end=" ")
    try:
        s1 = "{0:5s}: ".format(aufunter[rvd['prevmoonevent'][0]])
        s2 = dattims.round_datetime2minute(
            rvd['prevmoonevent'][1]).strftime(timefmt2)
        print(s1 + s2, file=f, end="")
    except:
        print("     " + dummytime, file=f, end="")
    try:
        s1 = "{0:5s}: ".format(aufunter[rvd['nextmoonevent'][0]])
        s2 = dattims.round_datetime2minute(
            rvd['nextmoonevent'][1]).strftime(timefmt2)
        print(s1 + s2, file=f, end="")
    except:
        print("   " + dummytime, file=f, end="")
    try:
        print("{0:3.0f} %          ".format(rvd['moonphase']), file=f, end="")
    except:
        print(dummytime, file=f, end=" ")
    try:
        print((rvd['nextfullmoon']).strftime(datefmt2), file=f, end="")
    except:
        print(dummytime, file=f, end=" ")
    try:
        print((rvd['nextnewmoon']).strftime(datefmt2), file=f, end="")
    except:
        print(dummytime, file=f, end=" ")
    print(file=f)
    print(tablemoonl1, file=f)
    print("\n\n", file=f)

    #  M o o n   -   P o s i t i o n -  I l l u m i n a n c e
    uff1 = "3. Moon position and illuminance"
    print(uff1, file=f)
    print("\n", file=f)
    BL1 = "=============== "
    # BL1s = "--------------- "
    BL2 = '======== '
    # BL2s = '-------- '
    # border line
    print(BL1, end="", file=f)
    [print(d.strftime(BL2), end='',
           file=f) for d in pydatetimearray]
    print(file=f)
    # times
    print("Time [UTC]      ", end="", file=f)
    [print(d.strftime('   %H:%M '), end='',
           file=f) for d in pydatetimearray]
    print(file=f)
    # border line
    print(BL1, end="", file=f)
    [print(d.strftime(BL2), end='',
           file=f) for d in pydatetimearray]
    print(file=f)
    # moon altitude
    uff2 = "Moon alt. [deg] "
    print(uff2, end="", file=f)
    [print('   {0:5.0f} '.format(d), end='',
           file=f) for d in
     np.rad2deg(moondata['altitude'])]
    print(file=f)
    print("Azimuth   [deg] ", end="", file=f)
    [print('   {0:5.0f} '.format(d), end='',
           file=f) for d in
     np.rad2deg(moondata['azimuth'])]
    print(file=f)
    # Rel. Illuminance
    print("rel. illumnin.  ", end="", file=f)
    [print('   {0:3.0f} % '.format(d), end='',
           file=f) for d in
     moondata['rel_illuminance_ESAA2013'] * 100.]
    print(file=f)
    # times PREVAILING
    print("**Prevailing**  ", end="", file=f)
    [print(d.strftime(' *%H:%M* '), end='',
           file=f) for d in pydatetimearray]
    print(file=f)
    # cloudiness N
    uff3 = " Clouds   N/{:2.0f}  ".format(constants.CLOUDAMOUNT_MAX)
    print(uff3, end="", file=f)
    [print('     {0:3s} '.format(d), end='',
           file=f) for d in
     parameters['cloudsn']]
    # ALLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
    # print(parameters)
    # BLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
    print(file=f)
    # cloud density
    print("Cloud density   ", end="", file=f)
    [print('  {0:6s} '.format(wolkendichte[d]), end='',
           file=f) for d in
     parameters['cloudsd']]
    print(file=f)
    # state of ground
    print("State of ground", end="", file=f)
    # print('-----------> inout.rst..parameters[ground]', parameters['ground'])
    # print('-----------> inout.rst..parameters[ground]', bodenzustand)
    [print('  {0:>7s}'.format(bodenzustand[d.upper()]), end='',
           file=f) for d in
     parameters['ground']]
    print(file=f)
    # Illuminance
    print("Illumin. [mlx] ", end="", file=f)
    [print('     {0:4s}'.format(format_illum_floats(d * 1000.)), end='', file=f)
        for d in illumdata['totalilluminance']]
    print(file=f)
    # times MINIMUM
    print("**Minimum**    ", end="", file=f)
    [print(d.strftime(' *%H:%M* '), end='',
           file=f) for d in pydatetimearray]
    print(file=f)
    # cloudiness N
    print(uff3, end="", file=f)
    [print('     {0:3s} '.format(d), end='',
           file=f) for d in
     parameters['cloudsnx']]
    print(file=f)
    # cloud density
    print("Wolken-Dichte   ", end="", file=f)
    # ALLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
    print('--->> In adnite2.inout: \n', parameters['cloudsdx'])
    # BLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
    [print('  {0:6s} '.format(wolkendichte[d]), end='', file=f)
     for d in parameters['cloudsdx']]
    print(file=f)
    # state of ground
    print("Bodenzustand   ", end="", file=f)
    [print('  {0:>7s}'.format(bodenzustand[d.upper()]), end='',
           file=f) for d in
     parameters['groundx']]
    print(file=f)
    # Illuminance
    print("Beleucht.[mlx] ", end="", file=f)
    [print('     {0:4s}'.format(format_illum_floats(d * 1000.)), end='', file=f)
        for d in illumdata['totalilluminancex']]
    print(file=f)
    # border line
    print(BL1, end="", file=f)
    [print(d.strftime(BL2), end='',
           file=f) for d in pydatetimearray]
    print(file=f)
    # end of output
    print("\n\n", file=f)
    f.close()
    return rsttablen


if __name__ == '__main__':
    version.version('Module')
    # deepdirs = '/home/thl/Test/this/is/very/deep'
    # print(create_dirs(deepdirs))
